/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision$
 */ 

/** @file
*/

/** @defgroup hal_io hal_io
 * @{
 * @ingroup hal
 * @brief Macros input/output management
*/


#ifndef HAL_IO_H__
#define HAL_IO_H__

#include "gpio.h"
#define HAL_IO_P00 0, 0
#define HAL_IO_P01 0, 1
#define HAL_IO_P02 0, 2
#define HAL_IO_P03 0, 3
#define HAL_IO_P04 0, 4
#define HAL_IO_P05 0, 5
#define HAL_IO_P06 0, 6
#define HAL_IO_P07 0, 7
#define HAL_IO_P08 0, 8
#define HAL_IO_P09 0, 9
#define HAL_IO_P010 0, 10
#define HAL_IO_P011 0, 11

#define HAL_IO_P10 1, 0
#define HAL_IO_P11 1, 1
#define HAL_IO_P12 1, 2
#define HAL_IO_P13 1, 3
#define HAL_IO_P14 1, 4
#define HAL_IO_P15 1, 5
#define HAL_IO_P16 1, 6
#define HAL_IO_P17 1, 7
#define HAL_IO_P18 1, 8
#define HAL_IO_P19 1, 9
#define HAL_IO_P110 1, 10
#define HAL_IO_P111 1, 11

#define HAL_IO_P20 2, 0
#define HAL_IO_P21 2, 1
#define HAL_IO_P22 2, 2
#define HAL_IO_P23 2, 3
#define HAL_IO_P24 2, 4
#define HAL_IO_P25 2, 5
#define HAL_IO_P26 2, 6
#define HAL_IO_P27 2, 7
#define HAL_IO_P28 2, 8
#define HAL_IO_P29 2, 9
#define HAL_IO_P210 2, 10
#define HAL_IO_P211 2, 11

#define HAL_IO_P30 3, 0
#define HAL_IO_P31 3, 1
#define HAL_IO_P32 3, 2
#define HAL_IO_P33 3, 3
#define HAL_IO_P34 3, 4
#define HAL_IO_P35 3, 5
#define HAL_IO_P36 3, 6
#define HAL_IO_P37 3, 7
#define HAL_IO_P38 3, 8
#define HAL_IO_P39 3, 9
#define HAL_IO_P310 3, 10
#define HAL_IO_P311 3, 11

#define HAL_IO_INPUT_DIR |= 
#define HAL_IO_OUTPUT_DIR &= ~

#define HAL_IO_OUTPUT 1
#define HAL_IO_INPUT 0

#define HAL_IO_OUTPUT_NORMAL_STRENGTH 0x00
#define HAL_IO_OUTPUT_HIGH_STRENGTH 0x60
#define HAL_IO_INPUT_BUF_ON_NO_PULL 0x00
#define HAL_IO_INPUT_BUF_ON_PULL_DOWN 0x20
#define HAL_IO_INPUT_BUF_ON_PULL_UP 0x40
#define HAL_IO_INPUT_BUF_OFF 0x60




/**@brief macro to configure an io.
 *  @details
 *  This macro configure a given io to input or output and with pullup/buffer configuration
 *  @param io_name io to configure
 *  @param is_input indicate if the io is to be set as an input
 *  @param io_mode pull resistor and buffer configuration (must be HAL_IO_OUTPUT_NORMAL_STRENGTH, HAL_IO_OUTPUT_HIGH_STRENGTH, HAL_IO_INPUT_BUF_ON_NO_PULL,
 *  HAL_IO_INPUT_BUF_ON_PULL_DOWN, HAL_IO_INPUT_BUF_ON_PULL_UP, or HAL_IO_INPUT_BUF_OFF)
*/
#define HAL_IO_CONFIG(io_name, is_input, io_mode) GPIOSetDir(io_name, is_input)

/**@brief macro to set an output.
 *  @details
 *  This macro set the given output to the given level (1 -> high, 0 -> low)
 *  @param io_name output to change
 *  @param io_state level to set.
*/
#define HAL_IO_SET_STATE(io_name, io_state) GPIOSetValue(io_name, io_state)

/**@brief macro to read the current state of an input.
 *  @details
 *  This macro reads the current state of a logical input
 *  @param io_name input to read
 *  @return input state level (1 if high, 0 if low)
*/
#define HAL_IO_READ(io_name) GPIOGetValue(io_name)


#endif //HAL_IO_H__

/** @} */
