/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision$
 */

#ifndef __LIB_ANT__H
#define __LIB_ANT__H

/** @file
* @brief ANT library
*/

/** @defgroup lib_ant lib_ant
@{
@ingroup lib

@brief Library for the ANT Message Protocol.

@details This is a wrapper around the ANT Message Protocol. It provides a
simplified API to the application. It is highly based on source code from
Dynastream Innovations, Inc.

*/

#include <stdint.h>
#include "bool.h"
#include "antdefines.h"
#include "antmessage.h"


/** @brief Initialization function
 *  @param baudrate Wanted baudrate
 */

 bool ANT_Init(unsigned short baudrate);

 /** @name Functions for configuration of the ANT module */
 //@{

 bool ANT_UnAssignChannel(uint8_t ucANTChannel); // Unassign a Channel

 bool ANT_AssignChannel(uint8_t ucANTChannel, uint8_t ucChanType, uint8_t ucNetNumber);

 bool ANT_AssignChannelExt(uint8_t ucAntChannel, uint8_t chan_type, uint8_t network, uint8_t ext);

 bool ANT_SetChannelId(uint8_t ucANTChannel, ushort usDeviceNumber, uint8_t ucDeviceType, uint8_t ucManufactureID);

 bool ANT_SetChannelPeriod(uint8_t ucANTChannel, ushort usMesgPeriod);

// bool ANT_SetChannelPeriodMultiplier(uint8_t ucANTChannel, USHORT usMesgPeriod, uint8_t ucANTChannelMultiplier);

 bool ANT_SetChannelSearchTimeout(uint8_t ucANTChannel, uint8_t ucSearchTimeout);   // Sets the search timeout for a give receive channel on the module

 bool ANT_SetChannelLowPrioritySearchTimeout(uint8_t ucANTChannel, uint8_t ucSearchTimeout);

 bool ANT_SetChannelRFFreq(uint8_t ucANTChannel, uint8_t ucRFFreq);

 bool ANT_SetFrequencyAgilitySettings(uint8_t ant_channel, uint8_t* freq_settings);

 bool ANT_SetNetworkKey(uint8_t ucNetNumber, uint8_t *pucKey);

 bool ANT_SetTransmitPower(uint8_t ucTransmitPower);

//@}

/** @name Test mode */
//@{

 bool ANT_InitCWTestMode(void);
 bool ANT_SetCWTestMode(uint8_t ucTransmitPower, uint8_t ucRFChannel);

//@}

 /** @name ANT Control messages */
 //@{

 bool ANT_ResetSystem(void);

 bool ANT_OpenChannel(uint8_t ucANTChannel); // Opens a Channel

 bool ANT_CloseChannel(uint8_t ucANTChannel); // Close a channel

 bool ANT_RequestMessage(uint8_t ucANTChannel, uint8_t ucMessageID);

 //@}


 /** @name Functions for updating the (synchronous) data sent over a channel */
 //@{

 bool ANT_SendBroadcastData(uint8_t ucANTChannel, uint8_t* pucData);   // Sends broadcast data to be sent on the channel's next synchronous message period
 bool ANT_SendAcknowledgedData(uint8_t ucANTChannel, uint8_t* pucData);  // Sends acknowledged data to be sent on the channel's next synchronous message period

 bool ANT_SendBurstTransferPacket(uint8_t channel, uint8_t* data, uint8_t size);  // Sends acknowledged data to be sent on the channel's next synchronous message period

//@}

 /** @brief Hook function for receiving messages from the ANT module.
  *  @details This function is called from the UART ISR.
  *  @param received_data Pointer to the received data.
  */

 void ANT_SerialHaveMessage(uint8_t *received_data);


 /** @name Helper functions */
 //@{
 uint8_t return_XOR(uint8_t *array, uint8_t length);
 uint8_t vol_return_XOR(volatile uint8_t *array, uint8_t length);
 bool checkMsg(uint8_t* msg);


uint8_t* lib_ant_get_burst_rx_buffer();

bool lib_ant_is_channel_open(uint8_t channel);

 //@}
#endif
