/*
 * main.c
 *
 * This is the startup code.
 */

#include <stdio.h>  // puts
#include <string.h>

#include "LPC11xx.h"
#include "system.h"
#include "gpio.h"
#include "lib_dispatcher.h"
#include "lib_aci.h"
#include "lib_ant.h"
#include "counter_app_ant.h"
#include "counter_app_btle.h"
#include "lpc_swu.h"
#include "timer32.h"

/*
 * This function is called by lib_aci when a valid message is
 * received. We use the dispatcher, so post a message there.
 */
void lib_aci_post_msg_hook(uint8_t ident)
{
   lib_dispatcher_post_msg(ident, NULL, 0, HIGH_PRIORITY);
}


int main(void)
{

   init_timer();			// Initialize timer
   swu_init(LPC_TMR32B1); 	// Initialize software UART. Used for the user interface
   system_init();			// Initialize GPIO, interrupts, etc.

   lib_aci_init();			// Initialize BLE library
   ANT_Init(19200);			// Initialize ANT library

   lib_dispatcher_init();	// Inilialize dispatcher

   reset_btle_radio();		// Hardware reset of radios
   reset_ant_radio();
   ANT_ResetSystem();		// ANT also needs a soft reset..

   // Set default setup values
   ant_setup.channel_type   = PARAMETER_TX_NOT_RX;
   ant_setup.channel_period = 4;
   ant_setup.frequencies[0] = 3;
   ant_setup.frequencies[1] = 39;
   ant_setup.frequencies[2] = 75;
   ant_setup.tx_power       = DEFAULT_RADIO_TX_POWER;	// default = 0 dBm
   ant_setup.search_timeout = 6;      // 6*2.5 = 15 sec

   btle_setup.tx_power    = ACI_DEVICE_OUTPUT_POWER_0DBM;
   btle_setup.adv_timeout = 15;


   while (1)
   {
      lib_dispatcher_dispatch();
   }

   return(0);
}
