/*
 * This is the ANT PC application. Based on an example application from Dynastream.
 *
 * Dynastream Innovations Inc.
 * Cochrane, AB, CANADA
 *
 * Copyright  1998-2009 Dynastream Innovations Inc.
 * All rights reserved. This software may not be reproduced by
 * any means without express written approval of Dynastream
 * Innovations Inc.
 */

#include "demo.h"

#include "types.h"
#include "dsi_framer_ant.hpp"
#include "dsi_thread.h"
#include "dsi_serial_generic.hpp"

#include <stdio.h>
#include <assert.h>
#include <string.h>

//#define ENABLE_EXTENDED_MESSAGES

#define USER_BAUDRATE         (57600)  // For AT3/AP2, use 57600
#define USER_RADIOFREQ        (35)

#define USER_ANTCHANNEL       (0)
#define USER_DEVICENUM        (1)
#define USER_DEVICETYPE       (2)
#define USER_TRANSTYPE        (3)

#define USER_NETWORK_KEY      {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,}
#define USER_NETWORK_NUM      (0)      // The network key is assigned to this network number

#define MESSAGE_TIMEOUT       (1000)

// Indexes into message recieved from ANT 
#define MESSAGE_BUFFER_DATA1_INDEX ((UCHAR) 0)
#define MESSAGE_BUFFER_DATA2_INDEX ((UCHAR) 1)
#define MESSAGE_BUFFER_DATA3_INDEX ((UCHAR) 2)
#define MESSAGE_BUFFER_DATA4_INDEX ((UCHAR) 3)
#define MESSAGE_BUFFER_DATA5_INDEX ((UCHAR) 4)
#define MESSAGE_BUFFER_DATA6_INDEX ((UCHAR) 5)
#define MESSAGE_BUFFER_DATA7_INDEX ((UCHAR) 6)
#define MESSAGE_BUFFER_DATA8_INDEX ((UCHAR) 7)
#define MESSAGE_BUFFER_DATA9_INDEX ((UCHAR) 8)
#define MESSAGE_BUFFER_DATA10_INDEX ((UCHAR) 9)
#define MESSAGE_BUFFER_DATA11_INDEX ((UCHAR) 10)
#define MESSAGE_BUFFER_DATA12_INDEX ((UCHAR) 11)
#define MESSAGE_BUFFER_DATA13_INDEX ((UCHAR) 12)
#define MESSAGE_BUFFER_DATA14_INDEX ((UCHAR) 13)


#define CLOCK_FREQUENCY (32768)


////////////////////////////////////////////////////////////////////////////////
// main
//
////////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{

	if (argc < 8)
	{
		printf("Usage: %s type period freq1 freq2 freq3 enable_freq_agility tx_power\n", argv[0]);
		return -1;
	}

	BYTE master_slave = (BYTE) atoi(argv[1]);
	USHORT period = (USHORT) atoi(argv[2]);
	BYTE freq1 = (UCHAR) atoi(argv[3]);
	BYTE freq2 = (UCHAR) atoi(argv[4]);
	BYTE freq3 = (UCHAR) atoi(argv[5]);
	BOOL en_freq_agility = (BOOL) atoi(argv[6]);
	BYTE tx_power = (BYTE) atoi(argv[7]);

	AntSetup* setup = new AntSetup(master_slave, period, freq1, freq2, freq3, en_freq_agility, tx_power, 4);
    Demo* pclDemo = new Demo(setup);

    UCHAR ucDeviceNumber = 0x00;

   if(pclDemo->Init())
      pclDemo->Start();
   else
      delete pclDemo;
   return 0;
}


AntSetup::AntSetup(BYTE type, USHORT period, BYTE freq1, BYTE freq2, BYTE freq3, bool freq_agility_enabled, BYTE tx_power, BYTE search_timeout)
{
	this->channel_type = type;
	this->channel_period = period;
	this->frequencies[0] = freq1;
	this->frequencies[1] = freq2;
	this->frequencies[2] = freq3;
	this->frequency_agility_enabled = freq_agility_enabled;
	this->tx_power = tx_power;
	this->search_timeout = search_timeout;
}

AntSetup::~AntSetup()
{
}


////////////////////////////////////////////////////////////////////////////////
// Demo
//
// Constructor, intializes Demo class
//
////////////////////////////////////////////////////////////////////////////////
Demo::Demo(AntSetup* setup)
{
   pclSerialObject = (DSISerialGeneric*)NULL;
   pclMessageObject = (DSIFramerANT*)NULL;
   uiDSIThread = (DSI_THREAD_ID)NULL;
   bMyDone = FALSE;
   bDone = FALSE;
   bDisplay = TRUE;
   bStarted = FALSE;

	counter = 0;
	received_packets = 0;
	sent_packets = 0;
	lost_packets = 0;
	failed_sent_packets = 0;
	retransmitted_rx_packets = 0;
	discovery_time = 0;
	invalid_counter = 0;

	ant_setup = setup;
	stop_cmd_sent = false;
	received_first_packet = false;
	connected = false;
   memset(aucTransmitBuffer,0,ANT_STANDARD_DATA_PAYLOAD_SIZE);
}

////////////////////////////////////////////////////////////////////////////////
// ~Demo
//
// Destructor, clean up and loose memory
//
////////////////////////////////////////////////////////////////////////////////
Demo::~Demo()
{
   if(pclMessageObject)
      delete pclMessageObject;
      
   if(pclSerialObject)
      delete pclSerialObject;

   if (ant_setup)
	   delete ant_setup;
}

////////////////////////////////////////////////////////////////////////////////
// Init
//
// Initize the Demo and ANT Library.
//
// ucDeviceNumber_: USB Device Number (0 for first USB stick plugged and so on)
//                  If not specified on command line, 0xFF is passed in as invalid.
// ucChannelType_:  ANT Channel Type. 0 = Master, 1 = Slave
//                  If not specified, 2 is passed in as invalid.
//
////////////////////////////////////////////////////////////////////////////////
BOOL Demo::Init()
{

   BOOL bStatus;

   // Initialize condition var and mutex
   UCHAR ucCondInit = DSIThread_CondInit(&condTestDone);
   assert(ucCondInit == DSI_THREAD_ENONE);

   UCHAR ucMutexInit = DSIThread_MutexInit(&mutexTestDone);
   assert(ucMutexInit == DSI_THREAD_ENONE);
   
#if defined(DEBUG_FILE)
   // Enable logging
   DSIDebug::Init();
   DSIDebug::SetDebug(TRUE);
#endif

   // Create Serial object.
   pclSerialObject = new DSISerialGeneric();
   assert(pclSerialObject);

   // NOTE: Will fail if the module is not available.
   // If no device number was specified on the command line, 
   // prompt the user for input.
   
   
   // Initialize Serial object.
   // The device number depends on how many USB sticks have been
   // plugged into the PC. The first USB stick plugged will be 0
   // the next 1 and so on.
   // 
   // The Baud Rate depends on the ANT solution being used. AP1
   // is 50000, all others are 57600   
   bStatus = pclSerialObject->Init(USER_BAUDRATE, 0x00);
   assert(bStatus);

   // Create Framer object.
   pclMessageObject = new DSIFramerANT(pclSerialObject);
   assert(pclMessageObject);

   // Initialize Framer object.
   bStatus = pclMessageObject->Init();
   assert(bStatus);

   // Let Serial know about Framer.
   pclSerialObject->SetCallback(pclMessageObject);

   // Open Serial.
   bStatus = pclSerialObject->Open();

   // If the Open function failed, most likely the device
   // we are trying to access does not exist, or it is connected
   // to another program
   if(!bStatus)
   {
      printf("Failed to connect to device at USB port 0\n");
      return FALSE;
   }

   // Create message thread.
   uiDSIThread = DSIThread_CreateThread(&Demo::RunMessageThread, this);
   assert(uiDSIThread);

   printf("Initialization was successful!\n"); fflush(stdout);

   return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
// Close
//
// Close connection to USB stick.
//
////////////////////////////////////////////////////////////////////////////////
void Demo::Close()
{   
	
   DSIThread_MutexLock(&mutexTestDone);
   bDone = TRUE;
   
   UCHAR ucWaitResult = DSIThread_CondTimedWait(&condTestDone, &mutexTestDone, DSI_THREAD_INFINITE);
   assert(ucWaitResult == DSI_THREAD_ENONE);
   
   DSIThread_MutexUnlock(&mutexTestDone);
   
   //Destroy mutex and condition var
   DSIThread_MutexDestroy(&mutexTestDone);
   DSIThread_CondDestroy(&condTestDone);
   
   //Close all stuff
   if(pclSerialObject)
      pclSerialObject->Close();

#if defined(DEBUG_FILE)
   DSIDebug::Close();
#endif
   printf("Exiting...\n");
   
}

////////////////////////////////////////////////////////////////////////////////
// Start 
//
// Starts the Demo
//
////////////////////////////////////////////////////////////////////////////////
void Demo::Start()
{
   BOOL bStatus;

   // Print out the menu to start
   //PrintMenu();   
   

   
   // Start ANT channel setup
   if (!InitANT())
   {
	   printf("failed to initialize ANT\n");
	   return;
   }

   UCHAR ucCmd[5];

	while (!bMyDone)
	{
		scanf("%s", &ucCmd[0]);
		if (strncmp((const char*)&ucCmd[0], "start", 5) == 0)
		{
			bStarted = TRUE;
			test_start_time = clock();
			// Open channel by setting network key. Channel setup is done as each setup command is successful
			printf("Opening ANT channel with:\n");
			printf("\tChannel type: %s\n", (ant_setup->channel_type == CHANNEL_TYPE_MASTER) ? "master" : "slave");
			printf("\tChannel period: %d\n", ant_setup->channel_period);
			printf("\tRadio frequencies: %d %d %d\n", ant_setup->frequencies[0], ant_setup->frequencies[1], ant_setup->frequencies[2]);
			printf("\tFreq agility: %s\n", ant_setup->frequency_agility_enabled ? "enabled" : "disabled");
			printf("\tTx power: %d\n", ant_setup->tx_power);

			
			UCHAR ucNetKey[8] = USER_NETWORK_KEY;
			pclMessageObject->SetNetworkKey(USER_NETWORK_NUM, ucNetKey, MESSAGE_TIMEOUT);
		}
		else if (strncmp((const char*)&ucCmd[0], "stop", 4) == 0)
		{
			stop_cmd_sent = true;
			pclMessageObject->CloseChannel(USER_ANTCHANNEL, MESSAGE_TIMEOUT);
		}
		else
		{
			break;
		}
		DSIThread_Sleep(0);
	}


	printf("Disconnecting module....\n");
	this->Close();
	printf("Test completed successfully\n");

   return;
}


////////////////////////////////////////////////////////////////////////////////
// InitANT
//
// Resets the system and starts the test
//
////////////////////////////////////////////////////////////////////////////////
BOOL Demo::InitANT(void)
{
   BOOL bStatus;
   
   // Reset system
   printf("Resetting module...\n");
   bStatus = pclMessageObject->ResetSystem();
   DSIThread_Sleep(1000);   
   
   
   return bStatus;
}

////////////////////////////////////////////////////////////////////////////////
// RunMessageThread 
//
// Callback function that is used to create the thread. This is a static 
// function.
//
////////////////////////////////////////////////////////////////////////////////
DSI_THREAD_RETURN Demo::RunMessageThread(void *pvParameter_)
{
   ((Demo*) pvParameter_)->MessageThread();
   return NULL;
}

////////////////////////////////////////////////////////////////////////////////
// MessageThread 
//
// Run message thread
////////////////////////////////////////////////////////////////////////////////
void Demo::MessageThread()
{
   ANT_MESSAGE stMessage;
   USHORT usSize;
   bDone = FALSE;

   while(!bDone)
   {
      if(pclMessageObject->WaitForMessage(1000 /*DSI_THREAD_INFINITE*/))
      {
         usSize = pclMessageObject->GetMessage(&stMessage);
         
         if(bDone)
            break;
         
         if(usSize == DSI_FRAMER_ERROR)
         {
            // Get the message to clear the error
            usSize = pclMessageObject->GetMessage(&stMessage, MESG_MAX_SIZE_VALUE);
            continue;
         }

         if(usSize != DSI_FRAMER_ERROR && usSize != DSI_FRAMER_TIMEDOUT && usSize != 0)
         {
            ProcessMessage(stMessage, usSize);
         }
      }
   }
   
   DSIThread_MutexLock(&mutexTestDone);
   UCHAR ucCondResult = DSIThread_CondSignal(&condTestDone);
   assert(ucCondResult == DSI_THREAD_ENONE);
   DSIThread_MutexUnlock(&mutexTestDone);
   //this->Close();
}


////////////////////////////////////////////////////////////////////////////////
// ProcessMessage 
//
// Process ALL messages that come from ANT, including event messages. 
//
// stMessage: Message struct containing message recieved from ANT
// usSize_:
////////////////////////////////////////////////////////////////////////////////
void Demo::ProcessMessage(ANT_MESSAGE stMessage, USHORT usSize_)
{
   BOOL bStatus;
   BOOL bPrintBuffer = FALSE;
   UCHAR ucDataOffset = MESSAGE_BUFFER_DATA2_INDEX;   // For most data messages

   static bool is_low_priority_search_timeout_set = false;

   if (!bStarted && stMessage.ucMessageID != MESG_STARTUP_MESG_ID) return;

   switch(stMessage.ucMessageID)
   {
      //RESPONSE MESG
      case MESG_RESPONSE_EVENT_ID:
      {
         //RESPONSE TYPE
		  if ((stMessage.aucData[1] != MESG_EVENT_ID) && (stMessage.aucData[2] != RESPONSE_NO_ERROR))
		  {
			  printf("cmd 0x%X failed with status 0x%X\n", stMessage.aucData[1], stMessage.aucData[2]);
			  if (stop_cmd_sent)
			  {
				  bMyDone = TRUE;
			  }
		  }
		  else 
		  {
			 switch(stMessage.aucData[1])
			 {
         
				case MESG_NETWORK_KEY_ID:
				{
				   //printf("Network Key set.\n\n");
				   //printf("Assigning channel...\n");
				   UCHAR channelType;
				   if (ant_setup->channel_type == CHANNEL_TYPE_MASTER) channelType = PARAMETER_TX_NOT_RX;
				   else if (ant_setup->channel_type == CHANNEL_TYPE_SLAVE) channelType = PARAMETER_RX_NOT_TX;
				   else 
				   {
					   printf("Error: invalid channel type\n");
					   return;
				   }

				   if (ant_setup->frequency_agility_enabled)
					{
						UCHAR msg[2];
						msg[0] = channelType;
						msg[1] = EXT_PARAM_FREQUENCY_AGILITY;
						bStatus = pclMessageObject->AssignChannelExt(USER_ANTCHANNEL, &msg[0], 2, 0, MESSAGE_TIMEOUT);
					}
					else
					{
						bStatus = pclMessageObject->AssignChannel(USER_ANTCHANNEL, channelType, 0, MESSAGE_TIMEOUT);
					}
				   break;
				}

				case MESG_ASSIGN_CHANNEL_ID:
				{
				   //printf("Channel Assigned\n\n");
				   //printf("Setting Channel ID...\n");
				   bStatus = pclMessageObject->SetChannelID(USER_ANTCHANNEL, USER_DEVICENUM, USER_DEVICETYPE, USER_TRANSTYPE, MESSAGE_TIMEOUT);
				   break;
				}

				case MESG_CHANNEL_ID_ID:
				{
				   //printf("Channel ID set\n\n");
				   //printf("Setting channel period...\n");
				   bStatus = pclMessageObject->SetChannelPeriod(USER_ANTCHANNEL, CLOCK_FREQUENCY/ant_setup->channel_period, MESSAGE_TIMEOUT);
				   break;
				}

				case MESG_CHANNEL_MESG_PERIOD_ID :
				{
					//printf("Channel period set\n\n");
					//printf("Setting LP search timout...\n");
					bStatus = pclMessageObject->SetLowPriorityChannelSearchTimeout(USER_ANTCHANNEL, 0, MESSAGE_TIMEOUT);
					break;
				}


				case MESG_SET_LP_SEARCH_TIMEOUT_ID :
				{
					//printf("LP search timeout set\n\n");
					//printf("Setting HP search timeout...\n");
					bStatus = pclMessageObject->SetChannelSearchTimeout(USER_ANTCHANNEL, 6, MESSAGE_TIMEOUT);
					break;
				}


				case MESG_CHANNEL_SEARCH_TIMEOUT_ID :
				{
					//printf("HP search timeout set\n\n");
					//printf("Setting channel tx power...\n");
					bStatus = pclMessageObject->SetChannelTransmitPower(USER_ANTCHANNEL, ant_setup->tx_power, MESSAGE_TIMEOUT);
					break;
				}

				


				case MESG_CHANNEL_RADIO_TX_POWER_ID :
				{
					//printf("Tx power set\n\n");
					//printf("Setting radio frequency.....\n");
					if (ant_setup->frequency_agility_enabled)
					{
						bStatus = pclMessageObject->ConfigFrequencyAgility(USER_ANTCHANNEL, ant_setup->frequencies[0], ant_setup->frequencies[1], ant_setup->frequencies[2], MESSAGE_TIMEOUT);
					}
					else
					{
						bStatus = pclMessageObject->SetChannelRFFrequency(USER_ANTCHANNEL, ant_setup->frequencies[0], MESSAGE_TIMEOUT);
					}
					break;
				}

				case MESG_CHANNEL_RADIO_FREQ_ID:
				{
				   //printf("Radio frequency set\n\n");
				   //printf("Opening channel...\n");
				   bStatus = pclMessageObject->OpenChannel(USER_ANTCHANNEL, MESSAGE_TIMEOUT);
				   break;
				}

				case MESG_AUTO_FREQ_CONFIG_ID :
				{
					//printf("Frequency agility configured\n\n");
					//printf("Opening channel...\n");
					bStatus = pclMessageObject->OpenChannel(USER_ANTCHANNEL, MESSAGE_TIMEOUT);
					break;
				}


				case MESG_OPEN_CHANNEL_ID:
				{
				   printf("Channel opened.\n\n");
				   break;
				}

				case MESG_UNASSIGN_CHANNEL_ID:
				{
					printf("Channel unassigned\n\n");
					printf("Test complete. Results: \n");
					printf("\tSent packets: %d\n", sent_packets);
					printf("\tReceived packets: %d\n", received_packets);
					printf("\tLost packets: %d\n", lost_packets);
					printf("\tFailed sent packets: %d\n", failed_sent_packets);
					printf("\tRetransmitted rx packets: %d\n", retransmitted_rx_packets);
					printf("\tRetransmitted tx packets: %d\n", failed_sent_packets);
					printf("\tPackets with invalid counter: %d\n", invalid_counter);
					printf("\tDiscovery time: %d\n", discovery_time);
					
					bMyDone = TRUE;
					break;
				}

				case MESG_CLOSE_CHANNEL_ID:
				{
					break;
				}


				case MESG_EVENT_ID:
				{
				   switch(stMessage.aucData[2])
					{
						case EVENT_CHANNEL_CLOSED:
						{
							if (stop_cmd_sent)
							{
								printf("Channel closed\n\n");
								printf("Unassigning channel...\n");
								bStatus = pclMessageObject->UnAssignChannel(USER_ANTCHANNEL, MESSAGE_TIMEOUT);
							}
							else
							{
								printf("ANT changed freqency/channel closed due to timeout\n");
							}
							break;
						}
					  case EVENT_TX:
					  {             
						 aucTransmitBuffer[0] = counter++;
						 //pclMessageObject->SendAcknowledgedData(USER_ANTCHANNEL, aucTransmitBuffer, MESSAGE_TIMEOUT);
						 pclMessageObject->SendBroadcastData(USER_ANTCHANNEL, aucTransmitBuffer);
						 if (ant_setup->frequency_agility_enabled == false)
						 {
							 sent_packets++;
						 }
						 break;
                  
					  }
					  case EVENT_RX_SEARCH_TIMEOUT:
					  {
						 printf("Search Timeout on channel\n"); 
						 break;
					  }
					  case EVENT_RX_FAIL:
					  {
						  if (connected)
						  {
							 printf("Rx Fail at %d ms\n", (clock() - test_start_time) * CLOCKS_PER_SEC/1000);
							 lost_packets++;
						  }
						 break;
					  }
					  case EVENT_TRANSFER_RX_FAILED:
					  {
						 printf("Burst recieve has failed\n");
						 break;
					  }
					  case EVENT_TRANSFER_TX_COMPLETED:     
					  {
						 aucTransmitBuffer[0] = counter++;
						 pclMessageObject->SendBroadcastData(USER_ANTCHANNEL, aucTransmitBuffer);
						 sent_packets++;
						 break;
					  }
					  case EVENT_TRANSFER_TX_FAILED:        
					  {
						 //aucTransmitBuffer[0] = counter++;
						 //pclMessageObject->SendAcknowledgedData(USER_ANTCHANNEL, aucTransmitBuffer, MESSAGE_TIMEOUT);
						 failed_sent_packets++;      
						 break;
					  }
					  case EVENT_RX_FAIL_GO_TO_SEARCH:      
					  {
						 printf("Goto Search.\n");
						 break;
					  }
					  case EVENT_CHANNEL_COLLISION:         
					  {
						 printf("Channel Collision\n");
						 break;
					  }
					  case EVENT_TRANSFER_TX_START:        
					  {
						 printf("Burst Started\n");      
						 break;
					  }
					  default:
					  {
								printf("Unknown Channel(%d) Event: 0x%X\n", stMessage.aucData[0], stMessage.aucData[2]); 
						 break;
					  }

				   }
               
				   break;
				}            

				default:
				{
						printf("Unknown Response 0x%X, Code 0%d\n", stMessage.aucData[1], stMessage.aucData[2]);
				   break;
				}
			 }
		}
		break;
      }
      
      case MESG_STARTUP_MESG_ID:
      {
         printf("RESET Complete, reason: ");
         
         UCHAR ucReason = stMessage.aucData[MESSAGE_BUFFER_DATA1_INDEX];
         
         if(ucReason == RESET_POR)
            printf("RESET_POR");
         
         if(ucReason & RESET_SUSPEND)
            printf("RESET_SUSPEND ");
         if(ucReason & RESET_SYNC)
            printf("RESET_SYNC ");
         if(ucReason & RESET_CMD)
            printf("RESET_CMD ");
         if(ucReason & RESET_WDT)
            printf("RESET_WDT ");
         if(ucReason & RESET_RST)
            printf("RESET_RST ");
            
         printf("\n");   
           
         break;
      }

      case MESG_ACKNOWLEDGED_DATA_ID:
	  {
		  byte counter_diff = stMessage.aucData[1] - counter;

		  if (stMessage.aucData[2] != 0)
		  {
			  printf("Disconnect time: %d\n", (clock()- test_start_time) * CLOCKS_PER_SEC/1000);
			  connected = false;
		  }
		  else
		  {
			  connected = true;
		  }

		  if (!received_first_packet)
		  {
			  discovery_time = (clock() - test_start_time) * CLOCKS_PER_SEC/1000;
		  }

		  if (counter_diff == 0 && received_first_packet)
		  {
				retransmitted_rx_packets++;
		  }
		  else if (counter_diff == 1)
		  {
				received_packets++;
		  }
		  else
		  {
			  invalid_counter++;
		  }
		  counter = stMessage.aucData[1];
		  received_first_packet = true;
		  break;
	  }
      case MESG_BURST_DATA_ID:  
	  {
			printf("Burst(0x%02x) Rx:(%d): ", ((stMessage.aucData[MESSAGE_BUFFER_DATA1_INDEX] & 0xE0) >> 5), stMessage.aucData[MESSAGE_BUFFER_DATA1_INDEX] & 0x1F );
			break;
	  }
      case MESG_BROADCAST_DATA_ID:
      {
		  byte counter_diff = stMessage.aucData[1] - counter;
		  
		  if (stMessage.aucData[2] != 0)
		  {
			  printf("Disconnect time: %d\n", (clock()- test_start_time) * CLOCKS_PER_SEC/1000);
			  connected = false;
		  }
		  else
		  {
			  connected = true;
		  }

		  if (!received_first_packet)
		  {
			  discovery_time = (clock() - test_start_time) * CLOCKS_PER_SEC/1000;
		  }

		  if (counter_diff == 0 && received_first_packet)
		  {
			  retransmitted_rx_packets++;
		  }
		  else if (counter_diff == 1)
		  {
			  received_packets++;
		  }
		  else
		  {
			  invalid_counter++;
		  }
		  counter = stMessage.aucData[1];
		  received_first_packet = true;
		  break;
      }

      default:
      {
         break;
      }
   }

   fflush(stdout);
   return;
}
