/*
 * Dynastream Innovations Inc.
 * Cochrane, AB, CANADA
 *
 * Copyright  1998-2009 Dynastream Innovations Inc.
 * All rights reserved. This software may not be reproduced by
 * any means without express written approval of Dynastream
 * Innovations Inc.
 */

#ifndef _DEMO_H_
#define _DEMO_H_

#include "types.h"
#include "dsi_framer_ant.hpp"
#include "dsi_thread.h"
#include "dsi_serial_generic.hpp"
#include <time.h>

#define CHANNEL_TYPE_MASTER   (0)
#define CHANNEL_TYPE_SLAVE		(1)
#define CHANNEL_TYPE_INVALID	(2)


class AntSetup {
public:
	AntSetup(BYTE type, USHORT period, BYTE freq1, BYTE freq2, BYTE freq3, bool freq_agility_enabled, BYTE tx_power, BYTE search_timeout);
	virtual ~AntSetup();

	BYTE channel_type;
	USHORT channel_period;
	BYTE frequencies[3];
	bool frequency_agility_enabled;
	BYTE tx_power;
	BYTE search_timeout;
};

class Demo {
public:
   Demo(AntSetup* setup);
   virtual ~Demo();
   BOOL Init();
   void Start();
   void Close();

private:
   BOOL InitANT();
   
   //Starts the Message thread.
   static DSI_THREAD_RETURN RunMessageThread(void *pvParameter_);

   //Listens for a response from the module
   void MessageThread();
   //Decodes the received message
   void ProcessMessage(ANT_MESSAGE stMessage, USHORT usSize_);
   
      
   BOOL bBursting; //holds whether the bursting phase of the test has started
   BOOL bMyDone;
   BOOL bDone;
   DSISerialGeneric* pclSerialObject;
   DSIFramerANT* pclMessageObject;
   DSI_THREAD_ID uiDSIThread;
   DSI_CONDITION_VAR condTestDone;
   DSI_MUTEX mutexTestDone;

   BOOL bDisplay;
   BOOL bStarted;
   UCHAR aucTransmitBuffer[ANT_STANDARD_DATA_PAYLOAD_SIZE];

   AntSetup* ant_setup;

   BYTE counter;
   UINT received_packets;
   UINT sent_packets;
   UINT lost_packets;
   UINT failed_sent_packets;
   UINT retransmitted_rx_packets;
   UINT invalid_counter;

   bool stop_cmd_sent;
   bool received_first_packet;
   bool connected;
   
   clock_t test_start_time;
   clock_t discovery_time;

//	UCHAR pucData[USER_DATA_LENGTH];  //data to transfer in a burst
   
   
   
   
};

#endif //TEST_H
