﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using Nordicsemi;

namespace masterEmulatorTester
{
    class Program
    {
        private static byte packet_counter = 0, lost_packets = 0;
        private static int received_packets = 0;
        private static bool first_run = true;

        private static string channel_map = "1FFFFFFFFF";
        private static double connectionIntervalMs = 40;
        private static double connectionSearchTimeout = 15;
        private static byte tx_power = 0;

        static void Main(string[] args)
        {
            #region Parse arguments
            if (args.Length != 4)
            {
                Console.WriteLine("Usage: masterEmulatorTester channel_map connection_interval search_timeout tx_power");
                return;
            }

            channel_map = args[0];
            connectionIntervalMs = 1000.0/Double.Parse(args[1]);
            connectionSearchTimeout = Double.Parse(args[2]);
            tx_power = byte.Parse(args[3]);

            #endregion

            MasterEmulator master = new MasterEmulator();
            master.LogMessage += new EventHandler<ValueEventArgs<string>>(master_LogMessage);
            master.DataReceived += new EventHandler<PipeDataEventArgs>(master_DataReceived);
            master.Connected += new EventHandler<EventArgs>(master_Connected);
            master.Disconnected += new EventHandler<ValueEventArgs<DisconnectReason>>(master_Disconnected);

            

            #region pipesetup
            // Counter pipe
            BtUuid serviceUuid1 = new BtUuid(0x1234, BtUuid.BtBaseUuid); //Counter service
            PipeStore pipeStore = PipeStore.Remote;
            master.SetupAddService(serviceUuid1, pipeStore);

            BtUuid charDefUuid1 = new BtUuid(0x2345); //Counter characteristic
            int maxDataLength = 1;
            byte[] data = new byte[] { };
            master.SetupAddCharacteristicDefinition(charDefUuid1, maxDataLength, data);

            PipeType pipeType1 = PipeType.Receive;
            int pipeNumber1 = master.SetupAssignPipe(pipeType1);

            #endregion

            /*
            Stack<string> usbDevs = new Stack<string>(master.EnumerateUsb());
            if (usbDevs.Count == 0)
            {
                Console.WriteLine("No usb devices found");
                Console.ReadKey();
                Environment.Exit(0);
            }
            string usbDev0 = string.Empty;
            while (usbDevs.Count > 0)
            {
                string dev = usbDevs.Pop();
                if (dev.Contains("FLE"))
                    continue;
                usbDev0 = dev;
            }

            if (usbDev0 == string.Empty)
            {
                Console.WriteLine("Found no suitable USB device. Press a key to exit");
                Console.ReadKey();
                Environment.Exit(0);
            }
            */

            master.Open("006RYE3E");
            Console.WriteLine(string.Format("IsOpen: {0}", master.IsOpen.ToString()));
            master.Run();
            Console.WriteLine(string.Format("IsRunning: {0}", master.IsRunning.ToString()));

            //int scanDuration = 10;
            BtScanParameters scanParameters = new BtScanParameters();
            scanParameters.ScanIntervalMs = 10;
            scanParameters.ScanWindowMs = 10;


            // Ready to run device discovery
            Console.WriteLine("ready");
            while (Console.ReadLine().CompareTo("start") != 0)
                ;


            
            
            /*
            List<BtDevice> devices = new List<BtDevice>(master.DiscoverDevices(scanDuration, scanParameters));
            if (devices.Count == 0)
            {
                Console.WriteLine("No devices discovered");
            }

            */
            BtDeviceAddress addressToConnectTo = new BtDeviceAddress("36E2313993F5");
            

            /*
            foreach (BtDevice dev in devices)
            {
                Console.WriteLine(dev.DeviceAddress.Value);
                foreach (KeyValuePair<DeviceInfoType, string> devInfo in dev.DeviceInfo)
                {
                    if (devInfo.Key == DeviceInfoType.CompleteLocalName)
                    {
                        if (devInfo.Value == "Per Magnus")
                        {
                            addressToConnectTo = dev.DeviceAddress;
                            TimeSpan discoveryDuration = DateTime.Now - startDiscoveryTime;
                            Console.WriteLine("Discovery took " + discoveryDuration.TotalMilliseconds + " milliseconds");
                        }
                    }
                    Console.WriteLine(string.Format("{0}:{1}", devInfo.Key, devInfo.Value));
                }
            }
            */


            if (addressToConnectTo != null)
            {
                BtConnectionParameters connectionParameters = new BtConnectionParameters();
                connectionParameters.ConnectionIntervalMs = 7.5;
                connectionParameters.SlaveLatency = 0;
                connectionParameters.SupervisionTimeoutMs = connectionSearchTimeout * 1000.0;
                
                DateTime startDiscoveryTime = DateTime.Now;
                bool connectSuccess = master.Connect(addressToConnectTo, ConnectionMode.ConnectWithoutBonding, connectionParameters);


                //bool disconnectsuccess = master.Disconnect();

                //bool bondSuccess = master.Connect(addressToConnectTo, ConnectionMode.ConnectWithBonding, connectionParameters);
                Console.WriteLine(string.Format("IsConnected: {0}", master.IsConnected.ToString()));

                if (master.IsConnected)
                {
                    try
                    {
                        master.SetChannelMap(channel_map);
                        master.SetRadioTxPower(tx_power);
                        master.DiscoverPipes();
                        connectionParameters.ConnectionIntervalMs = connectionIntervalMs;
                        master.UpdateConnectionParameters(connectionParameters);
                        TimeSpan discoveryDuration = DateTime.Now - startDiscoveryTime;
                        Console.WriteLine("Discovery took " + discoveryDuration.TotalMilliseconds + " ms");

                        while (Console.ReadLine().CompareTo("stop") != 0) // Lets hope the thread blocks at Console.ReadLine()...
                            ;
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(e.ToString());
                        while (Console.ReadLine().CompareTo("stop") != 0)
                            ;

                    }

                    //master.Disconnect();
                    //Console.ReadKey();
                }
                else
                {
                    Console.WriteLine("Could not connect to device.");
                    while (Console.ReadLine().CompareTo("stop") != 0)
                        ;
                }
            }


            master.Reset();
            master.Close();
        }

        #region Hook functions
        static void master_Connected(object sender, EventArgs e)
        {
            string msg = "CONNECTED EVENT";
            Console.WriteLine(msg);
            Debug.WriteLine(msg);
        }

        static void master_Disconnected(object sender, ValueEventArgs<DisconnectReason> e)
        {
            /*
            string msg = "DISCONNECTED EVENT, REASON: " + e.Value.ToString();
            Console.WriteLine(msg);
            Debug.WriteLine(msg);
            */
            Console.WriteLine("Disconnected.");
            Console.WriteLine(String.Format("\tReceived packets: {0:d}", received_packets));
            Console.WriteLine(String.Format("\tLost packets: {0:d}", lost_packets));
        }

        static void master_DataReceived(object sender, PipeDataEventArgs e)
        {

            byte current_counter = e.PipeData[0];
            byte tmp_lost_packets = (byte)(current_counter - packet_counter - 1);
            if (!first_run && (tmp_lost_packets > 0))
            {
                Console.WriteLine(String.Format("Lost {0:d} packets", tmp_lost_packets));
                lost_packets += (byte)(tmp_lost_packets);
            }

            //Console.WriteLine(String.Format("Current counter: {0:d}", current_counter));

            packet_counter = current_counter;
            received_packets++;
            first_run = false;
            /*
            string msg = "DATA_RECEIVED_EVENT PIPENUMBER " + e.PipeNumber + " DATA " + BitConverter.ToString(e.PipeData);
            Console.WriteLine(msg);
            Debug.WriteLine(msg);
            */
            
        }

        static void master_LogMessage(object sender, ValueEventArgs<string> e)
        {
            if (e.Value.Contains("Received AttHandleValueNotification"))
                return;
            string msg = "[Log] " + e.Value;
            Console.WriteLine(msg);
            Debug.WriteLine(msg);
        }


        #endregion
    }
}
