import serial
import threading
import socket
import time
import logging
import signal
import sys


def SerialReceiveThread(serial, stop_event):
    log_file = open("motherboard_log.txt", "w")
    while stop_event.isSet() == False:
        line = serial.readline()
        log_file.write(line)
  
    log_file.close()
    serial.close()
    

class MotherboardTestController:
    serial_interface = 0
    serial_thread = 0
    serial_thread_event = 0
    
    def __init__(self):
        self._setupSerial()
        self._setupLogging()

       
    def _setupSerial(self):
        self.serial_interface = serial.Serial(port=0, baudrate=9600, bytesize=serial.EIGHTBITS, 
            parity=serial.PARITY_NONE, stopbits=serial.STOPBITS_ONE, timeout=10)
        self.serial_interface.close()  # why do I need this exactly??
        self.serial_interface.open()
    
    def _setupLogging(self):
        if self.serial_interface != 0: 
            self.serial_thread_event = threading.Event()
            self.serial_thread = threading.Thread(target=SerialReceiveThread, args=(self.serial_interface, self.serial_thread_event))
            self.serial_thread.start()
    

    def write_cmd(self, cmd):
        self.serial_interface.write(cmd)
        self.serial_interface.flush()

    def __del__(self):
        self.serial_thread_event.set()
        self.serial_thread.join(2)
        self.serial_thread_event.clear()
        print "Goodbye"


def sigint_handler(signal, frame):
    print "Ctrl-C. Aborting"
    sys.exit(1)
        
        
try:
    logging.basicConfig(level=logging.DEBUG)
    signal.signal(signal.SIGINT, sigint_handler)  
    
    testController = MotherboardTestController()
    
    listen_socket_fd = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    listen_socket_fd.bind(('', 12345))
    print "Listening for connections..."
    listen_socket_fd.listen(1)
    (socket_fd, addr) = listen_socket_fd.accept()
    print 'Connected to', addr
    cmd = ''
    while 1:
        try:
            cmd = socket_fd.recv(32)
            if not cmd:
                print 'Lost connection. Listening...'
                (socket_fd, addr) = listen_socket_fd.accept()
                print 'Connected to', addr
            else: 
                testController.write_cmd(cmd)
                print cmd
        except socket.error, (errno, msg):
            print 'Lost connection due to exception. Listening...'
            (socket_fd, addr) = listen_socket_fd.accept()
            print 'Connected to', addr

        except KeyboardInterrupt, k:
            print 'Ctrl-C! Exiting..'
            sys.exit(0)
          
except Exception, ex:
    logging.exception(ex)

socket_fd.close()    
print "Socket closed"
sys.exit(0)