import serial
import sniffer
import time
import socket
from subprocess import Popen, PIPE
import ctypes

def kill(pid):
    """kill function for Win32"""
    kernel32 = ctypes.windll.kernel32
    handle = kernel32.OpenProcess(1, 0, pid)
    return (0 != kernel32.TerminateProcess(handle, 0))

ANT_PC_APP_PATH = "ANT PC app\DEMO_LIB.exe"
BTLE_PC_APP_PATH = "BTLE PC app\masterEmulatorTester.exe"
ANT_PC_LOG_PATH = "ANT PC app log files"
BTLE_PC_LOG_PATH = "BTLE PC app log files"
HOST = "192.9.200.188"
PORT = 12345
        


    
        
class BTLETestParams:
    connection_interval = 4
    channel_map = 0x1FFFFFFFFF
    master_tx_power = 3          # 0 = -18dB, 1 = -12dB, 2 = -6dB, 3 = 0dB
    slave_tx_power = 3
    search_timeout = 15
    
    def __init__(self):
        connection_interval = 4
        channel_map = 0x1FFFFFFFFF
        master_tx_power = 3
        slave_tx_power = 3
        search_timeout = 15


class ANTTestParams:
    pc_channel_type = 1          # 0 is master, 1 is slave
    peripheral_channel_type = 0
    freqs = (3, 39, 75)
    enable_freq_agility = 1
    channel_period = 4
    peripheral_tx_power = 3      # 0 = -20dB, 1 = -10dB, 2 = -5dB, 3 = 0dB
    pc_tx_power = 3
    
    def __init__(self):
        pc_channel_type = 1
        peripheral_channel_type = 0
        freqs = (3, 39, 75)
        enable_freq_agility = 1
        channel_period = 4
        peripheral_tx_power = 3
        pc_tx_power = 3




class Test:
    
    test_duration = 60                   # test duration in seconds
    test_name = None
    
    ant_pc_app = None
    btle_pc_app = None
    
    sniffer_module = None

    ant_pc_log = None
    btle_pc_log = None
    motherboard_log = None

    sock_fd = None
    
    def __init__(self):
        self.sock_fd = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock_fd.connect((HOST, PORT))
        
    def __del__(self):
        if self.sock_fd:
            self.sock_fd.close()
        if self.ant_pc_log:
            self.ant_pc_log.close()
        if self.btle_pc_log:
            self.btle_pc_log.close()
        if self.sniffer_module:
            self.sniffer_module.TerminateConnectionToAutomationServer()

    
    def socket_write_cmd(self, cmd):
        self.sock_fd.send(cmd)
        time.sleep(0.5)
        
        
    def motherboard_ant_setup(self, ant_test_params):
        self.socket_write_cmd("ant_type %d\n" % ant_test_params.peripheral_channel_type)
        self.socket_write_cmd("ant_period %d\n" % ant_test_params.channel_period)
        self.socket_write_cmd("ant_freq %d %d %d\n" % (ant_test_params.freqs[0], ant_test_params.freqs[1], ant_test_params.freqs[2]))
        self.socket_write_cmd("ant_tx_power %d\n" % ant_test_params.peripheral_tx_power)
        self.socket_write_cmd("ant_enable_freq_agility %d\n" % ant_test_params.enable_freq_agility)
        print "Motherboard ANT setup complete!"
        
    def motherboard_btle_setup(self, btle_test_params):
        self.socket_write_cmd("btle_tx_power %d\n" % btle_test_params.slave_tx_power)
        self.socket_write_cmd("btle_adv_timeout %d\n" % btle_test_params.search_timeout)
        print "Motherboard BTLE setup complete!"
        
    
    def sniffer_setup(self):
        print "Setting up sniffer.."
        self.sniffer_module = sniffer.Sniffer()
        self.sniffer_module.ConnectToAutomationServer()
        self.sniffer_module.StartFTS()               # FTS should be started manually before running this script
        self.sniffer_module.SendConfig()
        print "Sniffer setup complete!"

    def ant_pc_app_setup(self, ant_test_params):
        self.ant_pc_log = open(("%s\%s.txt" % (ANT_PC_LOG_PATH, self.test_name)), "w")
        ant_cmd = ("%s %d %d %d %d %d %d %d" % (ANT_PC_APP_PATH, ant_test_params.pc_channel_type, ant_test_params.channel_period, ant_test_params.freqs[0], ant_test_params.freqs[1], ant_test_params.freqs[2], ant_test_params.enable_freq_agility, ant_test_params.pc_tx_power)) 
        self.ant_pc_app = Popen(ant_cmd, bufsize=-1, stdin=PIPE, stdout=self.ant_pc_log, stderr=self.ant_pc_log, cwd="ANT PC app")
        print "ANT PC setup complete!"

    def btle_pc_app_setup(self, btle_test_params):
        self.btle_pc_log = open(("%s\%s.txt" % (BTLE_PC_LOG_PATH, self.test_name)), "w")
        btle_cmd = ("%s %x %d %d %d" % (BTLE_PC_APP_PATH, btle_test_params.channel_map, btle_test_params.connection_interval, btle_test_params.search_timeout, btle_test_params.master_tx_power))
        self.btle_pc_app = Popen(btle_cmd, bufsize=-1, stdin=PIPE, stdout=self.btle_pc_log, stderr=self.btle_pc_log, cwd="BTLE PC app")
        time.sleep(10)
        print "BTLE PC setup complete!"   
    
    def run_test(self, ant_test_params, btle_test_params):
        print "Starting test %s.." % self.test_name
        self.socket_write_cmd("reset\n")
        time.sleep(1)
        self.socket_write_cmd("=========\n%s\n=========\n" % self.test_name)
        
        if (btle_test_params != None):
            self.motherboard_btle_setup(btle_test_params)
            self.btle_pc_app_setup(btle_test_params)
            # Start sniffer capture
            if (self.sniffer_module != None):
                self.sniffer_module.StartCapture()
            self.btle_pc_app.stdin.write("start\r\n")
            self.btle_pc_app.stdin.flush()
            
            
        if (ant_test_params != None):
            self.motherboard_ant_setup(ant_test_params)
            self.ant_pc_app_setup(ant_test_params)
            self.ant_pc_app.stdin.write("start\r\n")
            self.ant_pc_app.stdin.flush()
            
        
        # Start motherboard application
        if ((ant_test_params != None) and (btle_test_params != None)):
            self.socket_write_cmd("start\n")
        elif (ant_test_params != None):
            self.socket_write_cmd("ant_start\n")
        elif (btle_test_params != None):
            self.socket_write_cmd("btle_start\n")

        # Wait until test is complete
        print "Test %s successfully started!" % self.test_name
        time.sleep(self.test_duration)
        print "Test complete!"
        
        # Transmissions are stopped by stopping the motherboard application
        self.socket_write_cmd("stop\n")
        
        
        # Stop PC applications. Dont exit if an exception is caught..
        try:
            if (ant_test_params != None):
                self.ant_pc_app.stdin.write("stop\r\n")
                self.ant_pc_app.stdin.flush()
                time.sleep(3)
                self.ant_pc_app.stdin.write("exit\r\n")
                self.ant_pc_app.stdin.flush()

            if (btle_test_params != None):
                self.btle_pc_app.stdin.write("stop\r\n")
                self.btle_pc_app.stdin.flush()
                time.sleep(3)
            if (self.sniffer_module != None):
                self.sniffer_module.StopCapture()
                self.sniffer_module.SaveCapture("%s" % self.test_name)
                self.sniffer_module.ClearBuffer()
        except Exception, ex:
            print ex
            if (ant_test_params != None and self.ant_pc_app.poll() == None):
                kill(self.ant_pc_app.pid)
            if (btle_test_params != None and self.btle_pc_app.poll() == None):
                kill(self.btle_pc_app.pid)
                
