from pc_test import ANTTestParams, BTLETestParams, Test           
import sys
import logging
import signal

def sigint_handler(signal, frame):
    print "Ctrl-C. Aborting"
    sys.exit(1)
    

def RunANTBTLEDefaultTests():
    channel_periods = (1, 5, 10, 15, 20, 25, 30, 50)
    test = Test()
    ant_test_params = ANTTestParams()
    btle_test_params = BTLETestParams()
    
    test.test_duration = 60
    ant_test_params.freqs = (66, 0, 0)
    ant_test_params.enable_freq_agility = 0
    ant_test_params.pc_channel_type = 1
    ant_test_params.peripheral_channel_type = 0
    
    #test.sniffer_setup()
    for i in range(0, len(channel_periods)):
        test.test_name = "%s%d" % ("ant_btle_default_test_", i+1)
        if (channel_periods[i] < 30):
            btle_test_params.connection_interval = channel_periods[i]
        else:
            btle_test_params.connection_interval = 30
        ant_test_params.channel_period = channel_periods[i]
        test.run_test(ant_test_params, btle_test_params)
        
    ant_test_params.pc_channel_type = 0
    ant_test_params.peripheral_channel_type = 1
    
    for i in range(len(channel_periods)+1, 2*len(channel_periods)+1):
        test.test_name = "%s%d" % ("ant_btle_default_test_", i)
        if (channel_periods[i-len(channel_periods)-1] < 30):
            btle_test_params.connection_interval = channel_periods[i-len(channel_periods)-1]
        else:
            btle_test_params.connection_interval = 30
        ant_test_params.channel_period = channel_periods[i-len(channel_periods)-1]
        test.run_test(ant_test_params, btle_test_params)
            
    
    


def RunANTReferenceTests():
    channel_periods = (1, 5, 10, 15, 20, 25, 30, 50)
    test = Test()
    ant_test_params = ANTTestParams()
    
    ant_test_params.freqs = (66, 0, 0)
    ant_test_params.enable_freq_agility = 0
    ant_test_params.pc_channel_type = 1
    ant_test_params.peripheral_channel_type = 0

    for i in range(0, len(channel_periods)):
        test.test_name = "%s%d" % ("ant_reference_test_", i+1)
        ant_test_params.channel_period = channel_periods[i]
        test.run_test(ant_test_params, None)

    ant_test_params.pc_channel_type = 0
    ant_test_params.peripheral_channel_type = 1
    
    for i in range(len(channel_periods)+1, 2*len(channel_periods)+1):
        test.test_name = "%s%d" % ("ant_reference_test_", i)
        ant_test_params.channel_period = channel_periods[i-len(channel_periods)-1]
        test.run_test(ant_test_params, None)



def RunBTLEReferenceTests():
    channel_periods = (1, 5, 10, 15, 20, 25, 30)
    test = Test()
    btle_test_params = BTLETestParams()
    test.test_duration = 60

    for i in range(0, len(channel_periods)):
        test.test_name = "%s%d" % ("btle_reference_test_", i+1)
        btle_test_params.connection_interval = channel_periods[i]
        test.run_test(None, btle_test_params)
            

        
        
def RunANTBTLEStressTests():
    test = Test()
    test.test_duration = 60
    
    ant_test_params = ANTTestParams()
    btle_test_params = BTLETestParams()
    
    ant_test_params.channel_period = 50
    ant_test_params.freqs = (66, 0, 0)
    ant_test_params.enable_freq_agility = 0
    btle_test_params.connection_interval = 30
    btle_test_params.channel_map = 0x0040000000
    
    ant_channel_types = (0, 1, 1, 0, 0, 1, 1, 0)
    ant_tx_powers = (3, 3, 0, 0)
    btle_tx_powers = (0, 0, 3, 3)

    for i in range(0, 4):
        test.test_name = "%s%d" % ("ant_btle_stress_test_", i+1)
        ant_test_params.peripheral_channel_type = ant_channel_types[i*2]
        ant_test_params.pc_channel_type = ant_channel_types[i*2+1]
        ant_test_params.peripheral_tx_power = ant_tx_powers[i]
        ant_test_params.pc_tx_power = ant_tx_powers[i]
        btle_test_params.master_tx_power = btle_tx_powers[i]
        btle_test_params.slave_tx_power = btle_tx_powers[i]
        test.run_test(ant_test_params, btle_test_params)
                


def RunANTBTLEOptimizedTests():
    test = Test()
    test.test_duration = 60
    
    ant_test_params = ANTTestParams()
    btle_test_params = BTLETestParams()
    
    channel_periods = (1, 5, 10, 15, 20, 25, 30, 50)
    
    ant_test_params.freqs = (66, 0, 0)
    ant_test_params.enable_freq_agility = 0 # does not work correctly...
    #ant_test_params.peripheral_channel_type = 0
    #ant_test_params.pc_channel_type = 1
    btle_test_params.channel_map = 0x1F1FFFFFFF
    
    #test.sniffer_setup()
    #for i in range(0, len(channel_periods)):
        #test.test_name = "%s%d" % ("ant_btle_optimized_test_", i+1)
        #if (channel_periods[i] < 30):
            #btle_test_params.connection_interval = channel_periods[i]
        #else:
            #btle_test_params.connection_interval = 30
        #ant_test_params.channel_period = channel_periods[i]
        #test.run_test(ant_test_params, btle_test_params)
        
    ant_test_params.pc_channel_type = 0
    ant_test_params.peripheral_channel_type = 1
    
    for i in range(len(channel_periods)+1, 2*len(channel_periods)+1):
        test.test_name = "%s%d" % ("ant_btle_optimized_test_", i)
        if (channel_periods[i-len(channel_periods)-1] < 30):
            btle_test_params.connection_interval = channel_periods[i-len(channel_periods)-1]
        else:
            btle_test_params.connection_interval = 30
        ant_test_params.channel_period = channel_periods[i-len(channel_periods)-1]
        test.run_test(ant_test_params, btle_test_params)


def RunANTWiFiTests():
    channel_periods = (1, 5, 10, 15, 20, 25, 30, 50)
    test = Test()
    ant_test_params = ANTTestParams()
    
    ant_test_params.freqs = (46, 0, 0)
    ant_test_params.enable_freq_agility = 0
    ant_test_params.pc_channel_type = 1
    ant_test_params.peripheral_channel_type = 0

    for i in range(0, len(channel_periods)):
        test.test_name = "%s%d" % ("ant_wifi_test_", i+1)
        ant_test_params.channel_period = channel_periods[i]
        test.run_test(ant_test_params, None)

    ant_test_params.pc_channel_type = 0
    ant_test_params.peripheral_channel_type = 1
    
    for i in range(len(channel_periods)+1, 2*len(channel_periods)+1):
        test.test_name = "%s%d" % ("ant_wifi_test_", i)
        ant_test_params.channel_period = channel_periods[i-len(channel_periods)-1]
        test.run_test(ant_test_params, None)
       

def RunBTLEWiFiTests():
    channel_periods = (1, 5, 10, 15, 20, 25, 30)
    test = Test()
    btle_test_params = BTLETestParams()
    btle_test_params.channel_map = 0x1fffffffff
    test.test_duration = 60
    #test.sniffer_setup()
    for i in range(0, len(channel_periods)):
        test.test_name = "%s%d" % ("btle_wifi_test_", i+1)
        btle_test_params.connection_interval = channel_periods[i]
        test.run_test(None, btle_test_params)
        
       
       
def RunANTBTLEWiFiOptimizedTests():
    channel_periods = (1, 5, 10, 15, 20, 25, 30, 50)
    test = Test()
    btle_test_params = BTLETestParams()
    btle_test_params.channel_map = 0x1F1FC007FF
    
    ant_test_params = ANTTestParams()
    ant_test_params.freqs = (66, 0, 0)
    ant_test_params.enable_freq_agility = 0
    ant_test_params.pc_channel_type = 1
    ant_test_params.peripheral_channel_type = 0

    #test.sniffer_setup()
    for i in range(0, len(channel_periods)):
        test.test_name = "%s%d" % ("ant_btle_optimized_wifi_test_", i+1)
        ant_test_params.channel_period = channel_periods[i]
        if (channel_periods[i] < 30):
            btle_test_params.connection_interval = channel_periods[i]
        else:
            btle_test_params.connection_interval = 30
        test.run_test(ant_test_params, btle_test_params)

    ant_test_params.pc_channel_type = 0
    ant_test_params.peripheral_channel_type = 1        
    
    for i in range(len(channel_periods)+1, 2*len(channel_periods)+1):
        test.test_name = "%s%d" % ("ant_btle_optimized_wifi_test_", i)
        ant_test_params.channel_period = channel_periods[i-len(channel_periods)-1]
        if (channel_periods[i-len(channel_periods)-1] < 30):
            btle_test_params.connection_interval = channel_periods[i-len(channel_periods)-1]
        else:
            btle_test_params.connection_interval = 30
        test.run_test(ant_test_params, btle_test_params)
            
            
def RunANTBTLEWiFiStressTests():
    channel_periods = (1, 5, 10, 15, 20, 25, 30, 50)
    test = Test()
    btle_test_params = BTLETestParams()
    btle_test_params.channel_map = 0x1fffffffff
    
    ant_test_params = ANTTestParams()
    ant_test_params.freqs = (46, 0, 0)
    ant_test_params.enable_freq_agility = 0
    ant_test_params.pc_channel_type = 1
    ant_test_params.peripheral_channel_type = 0

    #test.sniffer_setup()
    #for i in range(0, len(channel_periods)):
        #test.test_name = "%s%d" % ("ant_btle_stress_wifi_test_", i+1)
        #ant_test_params.channel_period = channel_periods[i]
        #if (channel_periods[i] < 30):
            #btle_test_params.connection_interval = channel_periods[i]
        #else:
            #btle_test_params.connection_interval = 30
        #test.run_test(ant_test_params, btle_test_params)

    ant_test_params.pc_channel_type = 0
    ant_test_params.peripheral_channel_type = 1        
    
    for i in range(16, 2*len(channel_periods)+1):
        test.test_name = "%s%d" % ("ant_btle_stress_wifi_test_", i)
        ant_test_params.channel_period = channel_periods[i-len(channel_periods)-1]
        if (channel_periods[i-len(channel_periods)-1] < 30):
            btle_test_params.connection_interval = channel_periods[i-len(channel_periods)-1]
        else:
            btle_test_params.connection_interval = 30
        test.run_test(ant_test_params, btle_test_params)            

        
def RunANTBTLEWiFiMoreOptimizedTests():
    channel_periods = (1, 5, 10, 15, 20, 25, 30, 50)
    test = Test()
    btle_test_params = BTLETestParams()
    btle_test_params.channel_map = 0x1C0000003F
    
    ant_test_params = ANTTestParams()
    ant_test_params.freqs = (16, 0, 0)
    ant_test_params.enable_freq_agility = 0
    ant_test_params.pc_channel_type = 1
    ant_test_params.peripheral_channel_type = 0

    #test.sniffer_setup()
    for i in range(0, len(channel_periods)):
        test.test_name = "%s%d" % ("ant_btle_more_optimized_wifi_test_", i+1)
        ant_test_params.channel_period = channel_periods[i]
        if (channel_periods[i] < 30):
            btle_test_params.connection_interval = channel_periods[i]
        else:
            btle_test_params.connection_interval = 30
        test.run_test(ant_test_params, btle_test_params)

    ant_test_params.pc_channel_type = 0
    ant_test_params.peripheral_channel_type = 1        
    
    for i in range(len(channel_periods)+1, 2*len(channel_periods)+1):
        test.test_name = "%s%d" % ("ant_btle_more_optimized_wifi_test_", i)
        ant_test_params.channel_period = channel_periods[i-len(channel_periods)-1]
        if (channel_periods[i-len(channel_periods)-1] < 30):
            btle_test_params.connection_interval = channel_periods[i-len(channel_periods)-1]
        else:
            btle_test_params.connection_interval = 30
        test.run_test(ant_test_params, btle_test_params)
            
       
        
        
logging.basicConfig(level=logging.DEBUG)
signal.signal(signal.SIGINT, sigint_handler)

try:            
    #RunANTReferenceTests()
    #RunBTLEReferenceTests()
    #RunANTBTLEDefaultTests()
    #RunANTBTLEOptimizedTests()
    #RunANTBTLEStressTests()
    
    #RunANTWiFiTests()
    #RunBTLEWiFiTests()
    #RunANTBTLEWiFiStressTests()
    RunANTBTLEWiFiMoreOptimizedTests()
    
except Exception, ex:
    logging.exception(ex)
    sys.exit(1)

