/*
 * timer16.c
 *
 *  Created on: 11. apr. 2011
 *      Author: pebr
 *
 *  Library for using the 32-bit timer/counter 0 without external capture and match registers.
 *
 */

#include "timer32.h"
#include "LPC11xx.h"

void start_timer(uint32_t countdown_ms)
{
    LPC_TMR32B0->TCR = 0x02;		/* reset timer */
    LPC_TMR32B0->PR = ((SystemCoreClock/LPC_SYSCON->SYSAHBCLKDIV) / 1000); /* set prescaler to 1 ms */
    LPC_TMR32B0->MR0 = countdown_ms;
    LPC_TMR32B0->IR  = 0xff;		/* reset all interrrupts */
    LPC_TMR32B0->MCR = 0x05;		/* stop timer and interrput on MR0 */
    LPC_TMR32B0->TCR = 0x01;		/* start timer */
}

void init_timer()
{
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<9);	// Enable clock
    NVIC_EnableIRQ(TIMER_32_0_IRQn);		// Enable interrupt in NVIC
    NVIC_SetPriority(TIMER_32_0_IRQn, 1); 	// Set lower priority to this interrupt (for swuart to function properly)
}

void delay_ms(uint32_t delayIn_ms)
{
    LPC_TMR32B0->TCR = 0x02;		/* reset timer */
    LPC_TMR32B0->PR = ((SystemCoreClock/LPC_SYSCON->SYSAHBCLKDIV) / 1000); /* set prescaler to 1 ms */
    LPC_TMR32B0->MR0 = delayIn_ms;
    LPC_TMR32B0->IR  = 0xff;		/* reset all interrrupts */
    LPC_TMR32B0->MCR = 0x04;		/* stop timer on MR0 */
    LPC_TMR32B0->TCR = 0x01;		/* start timer */

    while(LPC_TMR32B0->TCR & 0x01);	/* busy-wait until timer stops */
}


uint32_t timer32_get_time()
{
	return LPC_TMR32B0->TC;
}
