/*
 * Dynastream Innovations Inc.
 * Cochrane, AB, CANADA
 *
 *
 * The source code provided with the ANT DLL may be used with ANT devices only and remain the 
 * copyrighted property of Dynastream Innovations Inc. The source code is provided on an "as-is" 
 * basis and as an accommodation, and therefore all warranties, representations, or guarantees of any kind
 * (whether express, implied or statutory) including, without limitation, warranties of merchantability, 
 * non-infringement, or fitness for a particular purpose, are specifically disclaimed.
 * 
 * Copyright (c) 2008 Dynastream Innovations Inc.
 * All rights reserved. This software may not be reproduced by
 * any means without express written approval of Dynastream
 * Innovations Inc.
 *
 */

#ifndef ANTDEFINES_H
#define ANTDEFINES_H

#include "../src/bool.h"

//////////////////////////////////////////////
// Clock Definition
//////////////////////////////////////////////
#define CLOCK_FREQUENCY                            ((ulong)32768)          // ANT system clock frequency

//////////////////////////////////////////////
// Radio TX Power Definitions
//////////////////////////////////////////////
#define RADIO_TX_POWER_MASK                        ((uchar)0x03)
#define RADIO_TX_POWER_MINUS20DB                   ((uchar)0x00)
#define RADIO_TX_POWER_MINUS10DB                   ((uchar)0x01)
#define RADIO_TX_POWER_MINUS5DB                    ((uchar)0x02)
#define RADIO_TX_POWER_0DB                         ((uchar)0x03)

//////////////////////////////////////////////
// Default System Definitions
//////////////////////////////////////////////
#define DEFAULT_CHANNEL_MESSAGE_FREQUENCY          ((ulong)4)
#define DEFAULT_CHANNEL_MESSAGE_PERIOD             ((ushort)(CLOCK_FREQUENCY / DEFAULT_CHANNEL_MESSAGE_FREQUENCY)) // 8192
#define DEFAULT_RADIO_TX_POWER                     RADIO_TX_POWER_0DB      // ANT default RF power
#define DEFAULT_RADIO_CHANNEL                      ((uchar)66)             // 2400MHz + 1MHz * Channel Number = 2466MHz
#define DEFAULT_RX_SEARCH_TIMEOUT                  ((uchar)12)             // 12 * 2.5 seconds = 30 seconds

//////////////////////////////////////////////
// ID Definitions
//////////////////////////////////////////////
#define ANT_ID_SIZE                                ((uchar)4)
#define ANT_ID_TRANS_TYPE_OFFSET                   ((uchar)3)
#define ANT_ID_DEVICE_TYPE_OFFSET                  ((uchar)2)
#define ANT_ID_DEVICE_NUMBER_HIGH_OFFSET           ((uchar)1)
#define ANT_ID_DEVICE_NUMBER_LOW_OFFSET            ((uchar)0)
#define ANT_ID_DEVICE_TYPE_PAIRING_FLAG            ((uchar)0x80)

//////////////////////////////////////////////
// Assign Channel Parameters
//////////////////////////////////////////////
#define PARAMETER_RX_NOT_TX						   ((uchar)0x00)
#define PARAMETER_TX_NOT_RX                        ((uchar)0x10)
#define PARAMETER_SHARED_CHANNEL                   ((uchar)0x20)
#define PARAMETER_NO_TX_GUARD_BAND                 ((uchar)0x40)
#define PARAMETER_ALWAYS_RX_WILD_CARD_SEARCH_ID    ((uchar)0x40)
#define PARAMETER_RSSI_CHANNEL                     ((uchar)0x80)

//////////////////////////////////////////////
// Channel Status
//////////////////////////////////////////////
#define STATUS_CHANNEL_STATE_MASK                  ((uchar)0x03)
#define STATUS_UNASSIGNED_CHANNEL                  ((uchar)0x00)
#define STATUS_ASSIGNED_CHANNEL                    ((uchar)0x01)
#define STATUS_SEARCHING_CHANNEL                   ((uchar)0x02)
#define STATUS_TRACKING_CHANNEL                    ((uchar)0x03)

//////////////////////////////////////////////
// Standard capabilities defines
//////////////////////////////////////////////
#define CAPABILITIES_NO_RX_CHANNELS                ((uchar)0x01)
#define CAPABILITIES_NO_TX_CHANNELS                ((uchar)0x02)
#define CAPABILITIES_NO_RX_MESSAGES                ((uchar)0x04)
#define CAPABILITIES_NO_TX_MESSAGES                ((uchar)0x08)
#define CAPABILITIES_NO_ACKD_MESSAGES              ((uchar)0x10)
#define CAPABILITIES_NO_BURST_TRANSFER             ((uchar)0x20)

//////////////////////////////////////////////
// Advanced capabilities defines
//////////////////////////////////////////////
#define CAPABILITIES_OVERUN_UNDERRUN               ((uchar)0x01)     // Support for this functionality has been dropped
#define CAPABILITIES_NETWORK_ENABLED               ((uchar)0x02)
#define CAPABILITIES_AP1_VERSION_2                 ((uchar)0x04)     // This Version of the AP1 does not support transmit and only had a limited release
#define CAPABILITIES_SERIAL_NUMBER_ENABLED         ((uchar)0x08)
#define CAPABILITIES_PER_CHANNEL_TX_POWER_ENABLED  ((uchar)0x10)
#define CAPABILITIES_LOW_PRIORITY_SEARCH_ENABLED   ((uchar)0x20)
#define CAPABILITIES_SENSRCORE_ENABLED             ((uchar)0x40)
#define CAPABILITIES_SEARCH_LIST_ENABLED           ((uchar)0x80)

//////////////////////////////////////////////
// Advanced capabilities 2 defines
//////////////////////////////////////////////
#define CAPABILITIES_LED_ENABLED                   ((uchar)0x01)
#define CAPABILITIES_EXT_MESSAGE_ENABLED           ((uchar)0x02)
#define CAPABILITIES_SCAN_CHANNEL_ENABLED          ((uchar)0x04)
#define CAPABILITIES_TX_SEARCH_CMD_ENABLED         ((uchar)0x08)
// bits 4-7 reserved

//////////////////////////////////////////////
// Burst Message Sequence 
//////////////////////////////////////////////
#define CHANNEL_NUMBER_MASK                        ((uchar)0x1F)
#define SEQUENCE_NUMBER_MASK                       ((uchar)0xE0)
#define SEQUENCE_NUMBER_INC                        ((uchar)0x20)
#define SEQUENCE_NUMBER_ROLLOVER                   ((uchar)0x60)
#define SEQUENCE_LAST_MESSAGE                      ((uchar)0x80)

//////////////////////////////////////////////
// Response / Event Codes
//////////////////////////////////////////////
#define RESPONSE_NO_ERROR                          ((uchar)0x00)

#define EVENT_RX_SEARCH_TIMEOUT                    ((uchar)0x01)
#define EVENT_RX_FAIL                              ((uchar)0x02)
#define EVENT_TX                                   ((uchar)0x03)
#define EVENT_TRANSFER_RX_FAILED                   ((uchar)0x04)
#define EVENT_TRANSFER_TX_COMPLETED                ((uchar)0x05)
#define EVENT_TRANSFER_TX_FAILED                   ((uchar)0x06)
#define EVENT_CHANNEL_CLOSED                       ((uchar)0x07)
#define EVENT_RX_FAIL_GO_TO_SEARCH                 ((uchar)0x08)
#define EVENT_CHANNEL_COLLISION                    ((uchar)0x09)
#define EVENT_TRANSFER_TX_START                    ((uchar)0x0A)           // a pending transmit transfer has begun

#define EVENT_TRANSFER_TX_COMPLETED_RSSI           ((uchar)0x10)

#define CHANNEL_IN_WRONG_STATE                     ((uchar)0x15)           // returned on attempt to perform an action from the wrong channel state
#define CHANNEL_NOT_OPENED                         ((uchar)0x16)           // returned on attempt to communicate on a channel that is not open
#define CHANNEL_ID_NOT_SET                         ((uchar)0x18)           // returned on attempt to open a channel without setting the channel ID
#define CLOSE_ALL_CHANNELS                         ((uchar)0x19)           // returned when attempting to start scanning mode, when channels are still open

#define TRANSFER_IN_PROGRESS                       ((uchar)0x1F)           // returned on attempt to communicate on a channel with a TX transfer in progress
#define TRANSFER_SEQUENCE_NUMBER_ERROR             ((uchar)0x20)           // returned when sequence number is out of order on a Burst transfer
#define TRANSFER_IN_ERROR                          ((uchar)0x21)
#define TRANSFER_BUSY                              ((uchar)0x22)

#define INVALID_MESSAGE                            ((uchar)0x28)           // returned when the message has an invalid parameter
#define INVALID_NETWORK_NUMBER                     ((uchar)0x29)           // returned when an invalid network number is provided
#define INVALID_LIST_ID                            ((uchar)0x30)           // returned when the provided list ID or size exceeds the limit
#define INVALID_SCAN_TX_CHANNEL                    ((uchar)0x31)           // returned when attempting to transmit on channel 0 when in scan mode

#define NVM_FULL_ERROR                             ((uchar)0x40)           // error writing to NVM, memory is full
#define NVM_WRITE_ERROR                            ((uchar)0x41)           // error writing to NVM, bytes not written correctly

#define NO_RESPONSE_MESSAGE                        ((uchar)0x50)           // returned to the Command_SerialMessageProcess function, so no reply message is generated
#define RETURN_TO_MFG                              ((uchar)0x51)           // default return to any mesg when the module determines that the mfg procedure has not been fully completed


//////////////////////////////////////////////
// PC Application Event Codes
//////////////////////////////////////////////
//NOTE: These events are not generated by the embedded ANT module

#define EVENT_RX_BROADCAST                         ((uchar)0x9A)           // returned when module receives broadcast data
#define EVENT_RX_ACKNOWLEDGED                      ((uchar)0x9B)           // returned when module receives acknowledged data
#define EVENT_RX_BURST_PACKET                      ((uchar)0x9C)           // returned when module receives burst data

#define EVENT_RX_EXT_BROADCAST                     ((uchar)0x9D)           // returned when module receives broadcast data
#define EVENT_RX_EXT_ACKNOWLEDGED                  ((uchar)0x9E)           // returned when module receives acknowledged data
#define EVENT_RX_EXT_BURST_PACKET                  ((uchar)0x9F)           // returned when module receives burst data

#define EVENT_RX_RSSI_BROADCAST                    ((uchar)0xA0)           // returned when module receives broadcast data
#define EVENT_RX_RSSI_ACKNOWLEDGED                 ((uchar)0xA1)           // returned when module receives acknowledged data
#define EVENT_RX_RSSI_BURST_PACKET                 ((uchar)0xA2)           // returned when module receives burst data

#define EVENT_RX_BTH_BROADCAST                     ((uchar)0xA3)           // returned when module receives broadcast data
#define EVENT_RX_BTH_ACKNOWLEDGED                  ((uchar)0xA4)           // returned when module receives acknowledged data
#define EVENT_RX_BTH_BURST_PACKET                  ((uchar)0xA5)           // returned when module receives burst data

#define EVENT_RX_BTH_EXT_BROADCAST                 ((uchar)0xA6)           // returned when module receives broadcast data
#define EVENT_RX_BTH_EXT_ACKNOWLEDGED              ((uchar)0xA7)           // returned when module receives acknowledged data
#define EVENT_RX_BTH_EXT_BURST_PACKET              ((uchar)0xA8)           // returned when module receives burst data
//////////////////////////////////////////////
// NVM Command Codes
//////////////////////////////////////////////

#define NVM_CMD_FORMAT                             ((uchar)0x00)
#define NVM_CMD_DUMP                               ((uchar)0x01)
#define NVM_CMD_SET_DEFAULT_SECTOR                 ((uchar)0x02)
#define NVM_CMD_END_SECTOR                         ((uchar)0x03)
#define NVM_CMD_END_DUMP                           ((uchar)0x04)
#define NVM_CMD_LOCK_NVM                           ((uchar)0x05)

#endif // !ANTDEFINES_H                                                    


