/*
 * Dynastream Innovations Inc.
 * Cochrane, AB, CANADA
 *
 *
 * The source code provided with the ANT DLL may be used with ANT devices only and remain the 
 * copyrighted property of Dynastream Innovations Inc. The source code is provided on an "as-is" 
 * basis and as an accommodation, and therefore all warranties, representations, or guarantees of any kind
 * (whether express, implied or statutory) including, without limitation, warranties of merchantability, 
 * non-infringement, or fitness for a particular purpose, are specifically disclaimed.
 * 
 * Copyright (c) 2008 Dynastream Innovations Inc.
 * All rights reserved. This software may not be reproduced by
 * any means without express written approval of Dynastream
 * Innovations Inc.
 *
 */

#ifndef ANTMESSAGE_H
#define ANTMESSAGE_H

#include "bool.h"

/////////////////////////////////////////////////////////////////////////////
// Message Format
// Messages are in the format:
//
// AX XX YY -------- CK
//
// where: AX    is the 1 byte sync byte either transmit or recieve
//        XX    is the 1 byte size of the message (0-249) NOTE: THIS WILL BE LIMITED BY THE EMBEDDED RECEIVE BUFFER SIZE
//        YY    is the 1 byte ID of the message (1-255, 0 is invalid)
//        ----- is the data of the message (0-249 bytes of data)
//        CK    is the 1 byte Checksum of the message
/////////////////////////////////////////////////////////////////////////////
#define MESG_TX_SYNC                      ((uchar)0xA4)
#define MESG_RX_SYNC                      ((uchar)0xA5)
#define MESG_SIZE_OFFSET                  ((uchar)1)
#define MESG_ID_OFFSET                    ((uchar)2)
#define MESG_SYNC_SIZE                    ((uchar)1)
#define MESG_SIZE_SIZE                    ((uchar)1)
#define MESG_ID_SIZE                      ((uchar)1)
#define MESG_CHECKSUM_SIZE                ((uchar)1)
#define MESG_MAX_DATA_SIZE                ((uchar)17)
#define MESG_HEADER_SIZE                  (MESG_SYNC_SIZE + MESG_SIZE_SIZE + MESG_ID_SIZE)
#define MESG_DATA_OFFSET                  MESG_HEADER_SIZE  
#define MESG_FRAME_SIZE                   (MESG_HEADER_SIZE + MESG_CHECKSUM_SIZE)
#define MESG_MAX_SIZE                     (MESG_MAX_DATA_SIZE + MESG_FRAME_SIZE)

//////////////////////////////////////////////
// Message ID's
//////////////////////////////////////////////
#define MESG_INVALID_ID                   ((uchar)0x00)
#define MESG_EVENT_ID                     ((uchar)0x01)

#define MESG_VERSION_ID                   ((uchar)0x3E)  // protocol library version
#define MESG_RESPONSE_EVENT_ID            ((uchar)0x40)

#define MESG_UNASSIGN_CHANNEL_ID          ((uchar)0x41)
#define MESG_ASSIGN_CHANNEL_ID            ((uchar)0x42)
#define MESG_CHANNEL_MESG_PERIOD_ID       ((uchar)0x43)
#define MESG_CHANNEL_SEARCH_TIMEOUT_ID    ((uchar)0x44)
#define MESG_CHANNEL_RADIO_FREQ_ID        ((uchar)0x45)
#define MESG_NETWORK_KEY_ID               ((uchar)0x46)
#define MESG_RADIO_TX_POWER_ID            ((uchar)0x47)
#define MESG_RADIO_CW_MODE_ID             ((uchar)0x48)
#define MESG_SEARCH_WAVEFORM_ID           ((uchar)0x49)

#define MESG_SYSTEM_RESET_ID              ((uchar)0x4A)
#define MESG_OPEN_CHANNEL_ID              ((uchar)0x4B)
#define MESG_CLOSE_CHANNEL_ID             ((uchar)0x4C)
#define MESG_REQUEST_ID                   ((uchar)0x4D)

#define MESG_BROADCAST_DATA_ID            ((uchar)0x4E)
#define MESG_ACKNOWLEDGED_DATA_ID         ((uchar)0x4F)
#define MESG_BURST_DATA_ID                ((uchar)0x50)

#define MESG_CHANNEL_ID_ID                ((uchar)0x51)
#define MESG_CHANNEL_STATUS_ID            ((uchar)0x52)
#define MESG_RADIO_CW_INIT_ID             ((uchar)0x53)
#define MESG_CAPABILITIES_ID              ((uchar)0x54)

#define MESG_NVM_DATA_ID                  ((uchar)0x56)
#define MESG_NVM_CMD_ID                   ((uchar)0x57)
#define MESG_NVM_STRING_ID                ((uchar)0x58)

#define MESG_ID_LIST_ADD_ID               ((uchar)0x59)
#define MESG_ID_LIST_CONFIG_ID            ((uchar)0x5A)
#define MESG_OPEN_RX_SCAN_ID              ((uchar)0x5B)

#define MESG_EXT_CHANNEL_RADIO_FREQ_ID    ((uchar)0x5C)
#define MESG_EXT_BROADCAST_DATA_ID        ((uchar)0x5D)
#define MESG_EXT_ACKNOWLEDGED_DATA_ID     ((uchar)0x5E)
#define MESG_EXT_BURST_DATA_ID            ((uchar)0x5F)

#define MESG_CHANNEL_RADIO_TX_POWER_ID    ((uchar)0x60)
#define MESG_GET_SERIAL_NUM_ID            ((uchar)0x61)
#define MESG_GET_TEMP_CAL_ID              ((uchar)0x62)
#define MESG_SET_LP_SEARCH_TIMEOUT_ID     ((uchar)0x63)
#define MESG_SET_TX_SEARCH_ON_NEXT_ID     ((uchar)0x64)
#define MESG_SERIAL_NUM_SET_CHANNEL_ID_ID ((uchar)0x65)
#define MESG_RX_EXT_MESGS_ENABLE_ID       ((uchar)0x66)
#define MESG_RADIO_CONFIG_ALWAYS_ID       ((uchar)0x67)
#define MESG_ENABLE_LED_FLASH_ID          ((uchar)0x68)

#define MESG_AGC_CONFIG_ID                ((uchar)0x6A)

#define MESG_SYSTEM_STARTUP				  ((uchar)0x6F)

#define MESG_CONFIG_FREQ_AGILITY_ID		  ((uchar)0x70)

#define MESG_READ_SEGA_ID                 ((uchar)0xA0)
#define MESG_SEGA_CMD_ID                  ((uchar)0xA1)
#define MESG_SEGA_DATA_ID                 ((uchar)0xA2)
#define MESG_SEGA_ERASE_ID                ((uchar)0XA3)
#define MESG_SEGA_WRITE_ID                ((uchar)0XA4)
//                                        ((uchar)0xA5) //FREE
#define MESG_SEGA_LOCK_ID                 ((uchar)0xA6)
#define MESG_FUSECHECK_ID                 ((uchar)0xA7)
#define MESG_UARTREG_ID                   ((uchar)0XA8)
#define MESG_MAN_TEMP_ID                  ((uchar)0xA9)
#define MESG_BIST_ID                      ((uchar)0XAA)
#define MESG_SELFERASE_ID                 ((uchar)0XAB)
#define MESG_SET_MFG_BITS_ID              ((uchar)0xAC)
#define MESG_UNLOCK_INTERFACE_ID          ((uchar)0xAD)

#define MESG_IO_STATE_ID                  ((uchar)0xB0)
#define MESG_CFG_STATE_ID                 ((uchar)0xB1)

#define MESG_RSSI_ID                      ((uchar)0xC0)
#define MESG_RSSI_BROADCAST_DATA_ID       ((uchar)0xC1)
#define MESG_RSSI_ACKNOWLEDGED_DATA_ID    ((uchar)0xC2)
#define MESG_RSSI_BURST_DATA_ID           ((uchar)0xC3)
#define MESG_RSSI_SEARCH_THRESHOLD_ID     ((uchar)0xC4)

#define MESG_BTH_BROADCAST_DATA_ID        ((uchar)0xD0)
#define MESG_BTH_ACKNOWLEDGED_DATA_ID     ((uchar)0xD1)
#define MESG_BTH_BURST_DATA_ID            ((uchar)0xD2)

#define MESG_BTH_EXT_BROADCAST_DATA_ID    ((uchar)0xD3)
#define MESG_BTH_EXT_ACKNOWLEDGED_DATA_ID ((uchar)0xD4)
#define MESG_BTH_EXT_BURST_DATA_ID        ((uchar)0xD5)

//////////////////////////////////////////////
// Message Sizes
//////////////////////////////////////////////
#define MESG_INVALID_SIZE                 ((uchar)0)

#define MESG_RESPONSE_EVENT_SIZE          ((uchar)3)
#define MESG_CHANNEL_STATUS_SIZE          ((uchar)2)
#define MESG_VERSION_SIZE                 ((uchar)9)

#define MESG_UNASSIGN_CHANNEL_SIZE        ((uchar)1)
#define MESG_ASSIGN_CHANNEL_SIZE          ((uchar)3)
#define MESG_CHANNEL_ID_SIZE              ((uchar)5)
#define MESG_CHANNEL_MESG_PERIOD_SIZE     ((uchar)3)
#define MESG_CHANNEL_SEARCH_TIMEOUT_SIZE  ((uchar)2)
#define MESG_CHANNEL_RADIO_FREQ_SIZE      ((uchar)2)
#define MESG_NETWORK_KEY_SIZE             ((uchar)9)
#define MESG_RADIO_TX_POWER_SIZE          ((uchar)2)
#define MESG_RADIO_CW_MODE_SIZE           ((uchar)3)
#define MESG_RADIO_CW_INIT_SIZE           ((uchar)1)
#define MESG_SEARCH_WAVEFORM_SIZE         ((uchar)3)

#define MESG_SYSTEM_RESET_SIZE            ((uchar)1)
#define MESG_OPEN_CHANNEL_SIZE            ((uchar)1)
#define MESG_CLOSE_CHANNEL_SIZE           ((uchar)1)
#define MESG_REQUEST_SIZE                 ((uchar)2)
#define MESG_CAPABILITIES_SIZE            ((uchar)6)

#define MESG_DATA_SIZE                    9

#define MESG_NVM_DATA_SIZE                ((uchar)10)
#define MESG_NVM_CMD_SIZE                 ((uchar)3)
#define MESG_NVM_STRING_SIZE              ((uchar)9)

#define MESG_ID_LIST_ADD_SIZE             ((uchar)6)
#define MESG_ID_LIST_CONFIG_SIZE          ((uchar)3)
#define MESG_OPEN_RX_SCAN_SIZE            ((uchar)1)
#define MESG_EXT_CHANNEL_RADIO_FREQ_SIZE  ((uchar)3)
#define MESG_EXT_DATA_SIZE                ((uchar)13)

#define MESG_RADIO_CONFIG_ALWAYS_SIZE     ((uchar)2)
#define MESG_RX_EXT_MESGS_ENABLE_SIZE     ((uchar)2)
#define MESG_SET_TX_SEARCH_ON_NEXT_SIZE   ((uchar)2)
#define MESG_SET_LP_SEARCH_TIMEOUT_SIZE   ((uchar)2)

#define MESG_SERIAL_NUM_SET_CHANNEL_ID_SIZE ((uchar)3)
#define MESG_ENABLE_LED_FLASH_SIZE        ((uchar)2)
#define MESG_GET_SERIAL_NUM_SIZE          ((uchar)4)
#define MESG_GET_TEMP_CAL_SIZE            ((uchar)4)

#define MESG_AGC_CONFIG_SIZE              ((uchar)2)

#define MESG_CONFIG_FREQ_AGILITY_SIZE	  ((uchar)4)

#define MESG_READ_SEGA_SIZE               ((uchar)2)
#define MESG_SEGA_CMD_SIZE                ((uchar)3)
#define MESG_SEGA_DATA_SIZE               ((uchar)10)
#define MESG_SEGA_ERASE_SIZE              ((uchar)0)
#define MESG_SEGA_WRITE_SIZE              ((uchar)3)
#define MESG_SEGA_LOCKED_SIZE             ((uchar)1)
#define MESG_SEGA_LOCK_SIZE               ((uchar)0)
#define MESG_FUSECHECK_SIZE               ((uchar)1)
#define MESG_UARTREG_SIZE                 ((uchar)2)
#define MESG_MAN_TEMP_SIZE                ((uchar)2)
#define MESG_BIST_SIZE                    ((uchar)6)
#define MESG_SELFERASE_SIZE               ((uchar)2)

#define MESG_SET_MFG_BITS_SIZE            ((uchar)2)
#define MESG_UNLOCK_INTERFACE_SIZE        ((uchar)1)

#define MESG_IO_STATE_SIZE                ((uchar)2)
#define MESG_CFG_STATE_SIZE               ((uchar)2)

#define MESG_RSSI_SIZE                    ((uchar)3)
#define MESG_RSSI_DATA_SIZE               ((uchar)17)
#define MESG_RSSI_SEARCH_THRESHOLD_SIZE   ((uchar)2)

#endif // !ANTMESSAGE_H
