/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
*
* The information contained herein is property of Nordic Semiconductor ASA.
* Terms and conditions of usage are described in detail in NORDIC
* SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
*
* Licensees are granted free, non-transferable use of the information. NO
* WARRENTY of ANY KIND is provided. This heading must NOT be removed from
* the file.
*
* $LastChangedRevision: 4808 $
*/ 

/** @file
@brief Implementation of the ACI library.
*/

#include <stdint.h>
#include "bool.h"
#include <string.h>
#include "aci.h"
#include "aci_cmds.h"
#include "aci_evts.h"
#include "aci_protocol_defines.h"
#include "acilib_defs.h"
#include "acilib_if.h"
#include "acilib_types.h"
#include "system.h"
#include "services.h"
#include "dispatcher_config.h"    /* if you don't use lib_dispatcher, define in this file (dispatcher_config.h) the values for the HANDLES used by lib_aci (see lib_aci.h)*/
//#include "hal_power.h"
#include "hal_aci_tl.h"
#include "lib_aci.h"
#include "assertions.h"


#ifndef HANDLE_EVT_RADIO_STARTED_SETUP_IS_DEFINED
#define HANDLE_EVT_RADIO_STARTED_SETUP                     NB_MAX_MSG + 20
#endif
#ifndef HANDLE_EVT_RADIO_STARTED_TEST_IS_DEFINED
#define HANDLE_EVT_RADIO_STARTED_TEST                      NB_MAX_MSG + 21
#endif
#ifndef HANDLE_EVT_RADIO_STARTED_STANDBY_IS_DEFINED
#define HANDLE_EVT_RADIO_STARTED_STANDBY                   NB_MAX_MSG + 22   
#endif
#ifndef HANDLE_EVT_RADIO_STARTED_UNKNOWN_STATE_IS_DEFINED
#define HANDLE_EVT_RADIO_STARTED_UNKNOWN_STATE             NB_MAX_MSG + 23
#endif                                                     
#ifndef HANDLE_TRANSACTION_FINISHED_IS_DEFINED             
#define HANDLE_TRANSACTION_FINISHED                        NB_MAX_MSG + 24
#endif                                                     
#ifndef HANDLE_EVT_ERROR_IS_DEFINED                        
#define HANDLE_EVT_ERROR                                   NB_MAX_MSG + 25
#endif                                                     
#ifndef HANDLE_EVT_RADIO_CMD_RESPONSE_IS_DEFINED           
#define HANDLE_EVT_RADIO_CMD_RESPONSE                      NB_MAX_MSG + 26
#endif                                                     
#ifndef HANDLE_EVT_RADIO_DEVICE_VERSION_IS_DEFINED         
#define HANDLE_EVT_RADIO_DEVICE_VERSION                    NB_MAX_MSG + 27
#endif                                                     
#ifndef HANDLE_EVT_RADIO_DATA_RECEIVED_IS_DEFINED          
#define HANDLE_EVT_RADIO_DATA_RECEIVED                     NB_MAX_MSG + 28
#endif                                                     
#ifndef HANDLE_EVT_RADIO_PIPE_STATUS_IS_DEFINED        
#define HANDLE_EVT_RADIO_PIPE_STATUS                      NB_MAX_MSG + 29
#endif                                                     
#ifndef HANDLE_EVT_RADIO_ADVERTISE_TIMEDOUT_IS_DEFINED     
#define HANDLE_EVT_RADIO_ADVERTISE_TIMEDOUT                NB_MAX_MSG + 30
#endif                                                                  
#ifndef HANDLE_EVT_RADIO_DISCONNECTED_IS_DEFINED                        
#define HANDLE_EVT_RADIO_DISCONNECTED                      NB_MAX_MSG + 31
#endif                                                                  
#ifndef HANDLE_EVT_RADIO_CREDIT_IS_DEFINED                              
#define HANDLE_EVT_RADIO_CREDIT                            NB_MAX_MSG + 32
#endif                                                                  
#ifndef HANDLE_EVT_RADIO_TIMING_IS_DEFINED                              
#define HANDLE_EVT_RADIO_TIMING                            NB_MAX_MSG + 33
#endif                                                                  
#ifndef HANDLE_EVT_RADIO_CONNECTED_IS_DEFINED                           
#define HANDLE_EVT_RADIO_CONNECTED                         NB_MAX_MSG + 34
#endif                                                                  
#ifndef HANDLE_EVT_RADIO_BOND_STATUS_IS_DEFINED                         
#define HANDLE_EVT_RADIO_BOND_STATUS                       NB_MAX_MSG + 35
#endif                                                                  
#ifndef HANDLE_EVT_RADIO_HW_ERROR_IS_DEFINED                          
#define HANDLE_EVT_RADIO_HW_ERROR                          NB_MAX_MSG + 36
#endif                                                                  
#ifndef HANDLE_EVT_RADIO_PIPE_ERROR_IS_DEFINED                          
#define HANDLE_EVT_RADIO_PIPE_ERROR                        NB_MAX_MSG + 37
#endif                                                                  
#ifndef HANDLE_EVT_RADIO_ECHO_IS_DEFINED                                
#define HANDLE_EVT_RADIO_ECHO                              NB_MAX_MSG + 38
#endif                                                                  
#ifndef HANDLE_EVT_RADIO_DATA_ACK_IS_DEFINED                            
#define HANDLE_EVT_RADIO_DATA_ACK                          NB_MAX_MSG + 39
#endif


static aci_device_operation_mode_t last_reset_mode;

#ifdef SETUP_MESSAGES_CONTENT
static hal_aci_data_t setup_msgs[NB_SETUP_MESSAGES] = SETUP_MESSAGES_CONTENT; // removed "code" keyword
#else
static hal_aci_data_t *setup_msgs = NULL; // removed "code" keyword
#endif

static hal_aci_data_t  msg_to_send;
static volatile bool is_send_buffer_free = true;
static hal_aci_data_t  *p_rcvd_evt;

#define LIB_ACI_DEFAULT_CREDIT_NUMBER 1
#define LIB_ACI_DEFAULT_CONNEXION_INTERVAL 20


#define PIPES_ARRAY_SIZE      (ACI_DEVICE_MAX_PIPES + 1)/8

void hal_aci_tl_msg_rcv_hook(hal_aci_data_t *p_rcvd_data);

static aci_cmd_opcode_t cur_transaction_cmd;
static aci_cmd_opcode_t memorized_transaction_cmd;
static aci_cmd_opcode_t memorized_rcvd_cmd;
static aci_evt_params_cmd_rsp_t evt_params_cmd_rsp;


static bool is_cmd_response_expected;
static uint8_t device_mode2handle(aci_device_operation_mode_t device_mode);

static uint16_t cx_rf_interval;  //rf_interval = cx_rf_interval * 1.25 ms Range:0x0006 to 0x0C80
static volatile uint8_t nb_available_credits, nb_available_credits_on_device_started;

static bool is_request_data_pending;

static volatile uint8_t pipes_open_bitmap[PIPES_ARRAY_SIZE];
static volatile uint8_t pipes_closed_bitmap[PIPES_ARRAY_SIZE];
static aci_status_code_t host_setup_state; 
static uint8_t cur_error_code;

void lib_aci_enable_transmission()
{
  hal_aci_tl_enable_transaction();
  //hal_aci_tl_on_ready_signal();
}

bool lib_aci_radio_reset()
{

	if (is_send_buffer_free)
	{
		is_send_buffer_free = false;
		acil_encode_baseband_reset(&(msg_to_send.buffer[0]));
		cur_transaction_cmd = ACI_CMD_BASEBAND_RESET;
		//is_cmd_response_expected = true;
		return (hal_aci_tl_send(&msg_to_send));
	}
	return (false);
}

bool lib_aci_is_pipe_available(uint8_t pipe)
{
  uint8_t byte_idx = 0;
  for (byte_idx = 0; (byte_idx < 8)&&(pipe > 8); byte_idx++)
  {
    pipe -= 8;
  }
  if ((pipes_open_bitmap[byte_idx] >> pipe)&0x01)
  {
    return(true);
  }
  return(false);
}

bool lib_aci_is_pipe_closed(uint8_t pipe)
{
  uint8_t byte_idx = 0;
  for (byte_idx = 0; (byte_idx < 8)&&(pipe > 8); byte_idx++)
  {
    pipe -= 8;
  }
  if ((pipes_closed_bitmap[byte_idx] >> pipe)&0x01)
  {
    return(true);
  }
  return(false);
}

bool lib_aci_is_discovery_finished()
{
  return(pipes_open_bitmap[0]&0x01);
}

bool lib_aci_is_host_setup_complete()
{
  if (ACI_STATUS_TRANSACTION_COMPLETE == host_setup_state )
  {
    return(true);
  }
  return(false);
}

void lib_aci_init()
{
  uint8_t i;

  hal_aci_tl_init();

  nb_available_credits = LIB_ACI_DEFAULT_CREDIT_NUMBER;
  is_cmd_response_expected = true;
  for (i=0; i < PIPES_ARRAY_SIZE; i++)
  {
    pipes_open_bitmap[i] = 0;
    pipes_closed_bitmap[i] = 0;
  }
  is_send_buffer_free = true;
  is_request_data_pending = false;
}

uint8_t lib_aci_get_nb_available_credits()
{
  return nb_available_credits;
}

uint16_t lib_aci_get_cx_interval_ms()
{
  uint16_t cx_rf_interval_ms;
  cx_rf_interval_ms = cx_rf_interval;
  cx_rf_interval_ms *= 125;
  cx_rf_interval_ms = cx_rf_interval_ms / 100;
  return cx_rf_interval_ms;
}


bool lib_aci_set_test_mode(aci_test_mode_change_t enter_exit_test_mode)
{
  aci_cmd_params_test_t aci_cmd_params_test;
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    aci_cmd_params_test.test_mode_change = enter_exit_test_mode;
    acil_encode_cmd_set_test_mode(&(msg_to_send.buffer[0]), &aci_cmd_params_test);
    last_reset_mode = ACI_DEVICE_TEST;
    cur_transaction_cmd = ACI_CMD_TEST;
    is_cmd_response_expected = false;
    return (hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

bool lib_aci_sleep()
{
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    acil_encode_cmd_sleep(&(msg_to_send.buffer[0]));
    is_cmd_response_expected = false;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

bool lib_aci_device_version()
{
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    acil_encode_cmd_get_device_version(&(msg_to_send.buffer[0]));
    cur_transaction_cmd = ACI_CMD_GET_DEVICE_VERSION;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

bool lib_aci_set_local_data(uint8_t pipe, uint8_t *p_value, uint8_t size)
{
  aci_cmd_params_set_local_data_t aci_cmd_params_set_local_data;

  ASSERT(ERROR_CODE_LIB_ACI_OVERFLOW,(size <= ACI_PIPE_DATA_MAX_LEN));
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    if (size > ACI_PIPE_DATA_MAX_LEN)
    {
      size = ACI_PIPE_DATA_MAX_LEN;
    }
    aci_cmd_params_set_local_data.tx_data.pipe_number = pipe;
    memcpy(&(aci_cmd_params_set_local_data.tx_data.aci_data[0]), p_value, size);
    acil_encode_cmd_set_local_data(&(msg_to_send.buffer[0]), &aci_cmd_params_set_local_data, size);
    cur_transaction_cmd = ACI_CMD_SET_LOCAL_DATA;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

bool lib_aci_send_setup_msg()
{
#ifdef SETUP_MESSAGES_CONTENT
  static uint8_t nb_setup_msg_sent = 0;
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    if (hal_aci_tl_send(&(setup_msgs[nb_setup_msg_sent])))
    {
      cur_transaction_cmd = ACIL_DECODE_EVT_GET_OPCODE(&(setup_msgs[nb_setup_msg_sent].buffer[0]));
      nb_setup_msg_sent++;
      if (nb_setup_msg_sent >= NB_SETUP_MESSAGES)
      {
        nb_setup_msg_sent = 0;
        return(true);
      }
    }
  }
#endif
  return (false);
}

bool lib_aci_connect(uint16_t run_timeout, uint16_t adv_interval)
{
  aci_cmd_params_connect_t aci_cmd_params_connect;
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    aci_cmd_params_connect.timeout = run_timeout;
    aci_cmd_params_connect.adv_interval = adv_interval;
    acil_encode_cmd_connect(&(msg_to_send.buffer[0]), &aci_cmd_params_connect);
    cur_transaction_cmd = ACI_CMD_CONNECT;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

bool lib_aci_disconnect(aci_disconnect_reason_t reason)
{
  aci_cmd_params_disconnect_t aci_cmd_params_disconnect;
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    aci_cmd_params_disconnect.reason = reason;
    acil_encode_cmd_disconnect(&(msg_to_send.buffer[0]), &aci_cmd_params_disconnect);
    cur_transaction_cmd = ACI_CMD_DISCONNECT;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

bool lib_aci_bond(uint16_t run_timeout, uint16_t adv_interval)
{
  aci_cmd_params_bond_t aci_cmd_params_bond;
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    aci_cmd_params_bond.timeout = run_timeout;
    aci_cmd_params_bond.adv_interval = adv_interval;
    acil_encode_cmd_bond(&(msg_to_send.buffer[0]), &aci_cmd_params_bond);
    cur_transaction_cmd = ACI_CMD_BOND;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

bool lib_aci_device_wakeup()
{
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    acil_encode_cmd_wakeup(&(msg_to_send.buffer[0]));
    cur_transaction_cmd = ACI_CMD_WAKEUP;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

bool lib_aci_set_radio_tx_power(aci_device_output_power_t tx_power)
{
  aci_cmd_params_set_tx_power_t aci_cmd_params_set_tx_power;
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    aci_cmd_params_set_tx_power.device_power = tx_power;
    acil_encode_cmd_set_radio_tx_power(&(msg_to_send.buffer[0]), &aci_cmd_params_set_tx_power);
    cur_transaction_cmd = ACI_CMD_SET_TX_POWER;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

bool lib_aci_get_address()
{
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    acil_encode_cmd_get_address(&(msg_to_send.buffer[0]));
    cur_transaction_cmd = ACI_CMD_GET_DEVICE_ADDRESS;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}


bool lib_aci_send_data(uint8_t pipe, uint8_t *p_value, uint8_t size)
{
  aci_cmd_params_send_data_t aci_cmd_params_send_data;

  ASSERT(ERROR_CODE_LIB_ACI_OVERFLOW,(size <= ACI_PIPE_DATA_MAX_LEN));
  if (size > ACI_PIPE_DATA_MAX_LEN)
  {
    size = ACI_PIPE_DATA_MAX_LEN;
  }
  if ((nb_available_credits > 0))
  {
    if (is_send_buffer_free)
    {
      is_send_buffer_free = false;
      nb_available_credits--;
      aci_cmd_params_send_data.tx_data.pipe_number = pipe;
      memcpy(&(aci_cmd_params_send_data.tx_data.aci_data[0]), p_value, size);
      acil_encode_cmd_send_data(&(msg_to_send.buffer[0]), &aci_cmd_params_send_data, size);
      is_cmd_response_expected = false;
      return(hal_aci_tl_send(&msg_to_send));
    }
  }
  return (false);
}

bool lib_aci_request_data(uint8_t pipe)
{
  aci_cmd_params_request_data_t aci_cmd_params_request_data;
  if ((nb_available_credits > 0)&&(!is_request_data_pending))
  {
    if (is_send_buffer_free)
    {
      nb_available_credits--;
      is_request_data_pending = true;
      is_send_buffer_free = false;
      aci_cmd_params_request_data.pipe_number = pipe;
      acil_encode_cmd_request_data(&(msg_to_send.buffer[0]), &aci_cmd_params_request_data);
      cur_transaction_cmd = ACI_CMD_REQUEST_DATA;
      is_cmd_response_expected = false;
      return(hal_aci_tl_send(&msg_to_send));
    }
  }
  return (false);
}

bool lib_aci_open_remote_pipe(uint8_t pipe)
{
  aci_cmd_params_open_remote_pipe_t aci_cmd_params_open_remote_pipe;
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    aci_cmd_params_open_remote_pipe.pipe_number = pipe;
    acil_encode_cmd_open_remote_pipe(&(msg_to_send.buffer[0]), &aci_cmd_params_open_remote_pipe);
    cur_transaction_cmd = ACI_CMD_OPEN_REMOTE_PIPE;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

bool lib_aci_echo_msg(uint8_t msg_size, uint8_t *p_msg_data)
{
  aci_cmd_params_echo_t aci_cmd_params_echo;
  ASSERT(ERROR_CODE_LIB_ACI_OVERFLOW,(msg_size <= (ACI_ECHO_DATA_MAX_LEN)));
  if (is_send_buffer_free)
  {
    is_send_buffer_free = false;
    if (msg_size > (ACI_ECHO_DATA_MAX_LEN))
    {
      msg_size = ACI_ECHO_DATA_MAX_LEN;
    }

    memcpy(&(aci_cmd_params_echo.echo_data[0]), p_msg_data, msg_size);
    acil_encode_cmd_echo_msg(&(msg_to_send.buffer[0]), &aci_cmd_params_echo, msg_size);

    cur_transaction_cmd = ACI_CMD_ECHO;
    return(hal_aci_tl_send(&msg_to_send));
  }
  return (false);
}

static uint8_t device_mode2handle(aci_device_operation_mode_t device_mode)
{
  switch (device_mode)
  {
  case ACI_DEVICE_SETUP :
    return(HANDLE_EVT_RADIO_STARTED_SETUP);
  case ACI_DEVICE_TEST :
    return(HANDLE_EVT_RADIO_STARTED_TEST);
  case ACI_DEVICE_STANDBY :
    return(HANDLE_EVT_RADIO_STARTED_STANDBY);
  }
  return(HANDLE_EVT_RADIO_STARTED_UNKNOWN_STATE);
}

void lib_aci_interpret_evt_cmd_rsp(aci_evt_params_cmd_rsp_t *p_out_data)
{
  acil_decode_evt_command_response(&(p_rcvd_evt->buffer[0]), p_out_data);
}

void lib_aci_interpret_evt_device_started(aci_evt_params_device_started_t *p_out_data)
{
  acil_decode_evt_device_started(&(p_rcvd_evt->buffer[0]), p_out_data);
}

void lib_aci_interpret_evt_pipes_status(aci_evt_params_pipe_status_t *p_out_data)
{
  acil_decode_evt_pipe_status(&(p_rcvd_evt->buffer[0]), p_out_data);
}

void lib_aci_interpret_evt_disconnected(aci_evt_params_disconnected_t *p_out_data)
{
  acil_decode_evt_disconnected(&(p_rcvd_evt->buffer[0]), p_out_data);
}

uint8_t lib_aci_interpret_evt_data_rcvd(aci_evt_params_data_received_t *p_out_data)
{
  return acil_decode_evt_data_received(&(p_rcvd_evt->buffer[0]), p_out_data);
}

uint8_t lib_aci_interpret_evt_data_ack(void)
{
  aci_evt_params_data_ack_t evt_params_data_ack;
  acil_decode_evt_data_ack(&(p_rcvd_evt->buffer[0]), &evt_params_data_ack);
  return evt_params_data_ack.pipe_number;
}

uint8_t lib_aci_interpret_evt_hw_error(aci_evt_params_hw_error_t *p_out_data)
{
  return acil_decode_evt_hw_error(&(p_rcvd_evt->buffer[0]), p_out_data);
}

uint8_t lib_aci_interpret_evt_credit(void)
{
  aci_evt_params_data_credit_t evt_params_data_credit;
  acil_decode_evt_credit(&(p_rcvd_evt->buffer[0]), &evt_params_data_credit);
  return evt_params_data_credit.credit;
}

void lib_aci_interpret_evt_timing(aci_evt_params_timing_t *p_out_data)
{
  acil_decode_evt_timing(&(p_rcvd_evt->buffer[0]), p_out_data);
}

uint8_t lib_aci_interpret_evt_pipe_error(aci_evt_params_pipe_error_t *p_out_data) // was (void)
{
  //aci_evt_params_pipe_error_t evt_params_pipe_error;
  acil_decode_evt_pipe_error(&(p_rcvd_evt->buffer[0]), p_out_data);
  return p_out_data->pipe_number;
}

uint8_t lib_aci_interpret_evt_echo(uint8_t *p_out_data)
{
  return acil_decode_evt_echo(&(p_rcvd_evt->buffer[0]), p_out_data);
}

void lib_aci_interpret_evt_connected(aci_evt_params_connected_t *p_out_data)
{
  acil_decode_evt_connected(&(p_rcvd_evt->buffer[0]), p_out_data);
}

void lib_aci_interpret_evt_bond_status(aci_evt_params_bond_status_t *p_out_data)
{
  acil_decode_evt_bond_status(&(p_rcvd_evt->buffer[0]), p_out_data);
}

void lib_aci_interpret_error(lib_aci_error_descriptor_t *p_error_data)
{
  switch(cur_error_code)
  {
  case ERROR_CMD_RESPONSE_ERROR:
    p_error_data->error_code      = ERROR_CMD_RESPONSE_ERROR;
    p_error_data->error_sub_code1 = evt_params_cmd_rsp.cmd_opcode;
    p_error_data->error_sub_code2 = evt_params_cmd_rsp.cmd_status;
    break;
  case ERROR_CMD_RESPONSE_WRONG_CMD:
    p_error_data->error_code      = ERROR_CMD_RESPONSE_WRONG_CMD;
    p_error_data->error_sub_code1 = evt_params_cmd_rsp.cmd_opcode;
    p_error_data->error_sub_code2 = memorized_transaction_cmd;
    break;
  case ERROR_RADIO_EVT_UNKNOWN:
    p_error_data->error_code      = ERROR_RADIO_EVT_UNKNOWN;
    p_error_data->error_sub_code1 = ACIL_DECODE_EVT_GET_OPCODE(&(p_rcvd_evt->buffer[0]));
    break;
  case ERROR_CMD_RESPONSE_TOO_SHORT :
    p_error_data->error_code      = ERROR_CMD_RESPONSE_TOO_SHORT;
    p_error_data->error_sub_code1 = ACIL_DECODE_EVT_GET_LENGTH(&(p_rcvd_evt->buffer[0]));
    break;
  default :
    p_error_data->error_code      = cur_error_code;
  }
}

void hal_aci_tl_msg_rcv_hook(hal_aci_data_t *p_rcvd_data)
{
  uint8_t msg_handle;
  aci_evt_opcode_t received_opcode;
  uint8_t evt_length;
  
  if (!is_cmd_response_expected)
  {
    is_send_buffer_free = true;
    lib_aci_post_msg_hook(HANDLE_TRANSACTION_FINISHED);
  }
  is_cmd_response_expected = true;
  evt_length  = ACIL_DECODE_EVT_GET_LENGTH(&(p_rcvd_data->buffer[0]));
  if (evt_length > 0)
  {
    hal_aci_tl_disable_transaction();
    received_opcode = (aci_evt_opcode_t)ACIL_DECODE_EVT_GET_OPCODE(&(p_rcvd_data->buffer[0]));
    p_rcvd_evt = p_rcvd_data;
    msg_handle = HANDLE_EVT_ERROR;
    switch(received_opcode)
    {
    case ACI_EVT_CMD_RSP:
      {
        if (evt_length < EVT_CMD_RESPONSE_MIN_LENGTH)
        {
          cur_error_code = ERROR_CMD_RESPONSE_TOO_SHORT;
          msg_handle = HANDLE_EVT_ERROR;
        }
        else
        {
          acil_decode_evt_command_response(&(p_rcvd_evt->buffer[0]), &evt_params_cmd_rsp);

          msg_handle = HANDLE_EVT_RADIO_CMD_RESPONSE;
          if (ACI_CMD_GET_DEVICE_VERSION == evt_params_cmd_rsp.cmd_opcode)
          {
            msg_handle = HANDLE_EVT_RADIO_DEVICE_VERSION;
          }
          if (cur_transaction_cmd != evt_params_cmd_rsp.cmd_opcode)
          {
            cur_error_code = ERROR_CMD_RESPONSE_WRONG_CMD;
            msg_handle = HANDLE_EVT_ERROR;
            memorized_transaction_cmd = cur_transaction_cmd;
            memorized_rcvd_cmd = evt_params_cmd_rsp.cmd_opcode;
          }
          if ((ACI_STATUS_SUCCESS != evt_params_cmd_rsp.cmd_status)&&(ACI_CMD_WAKEUP != evt_params_cmd_rsp.cmd_opcode))
          {
            if (ACI_STATUS_TRANSACTION_CONTINUE == evt_params_cmd_rsp.cmd_status)
            {
              host_setup_state   = ACI_STATUS_TRANSACTION_CONTINUE;
            }
            else if (ACI_STATUS_TRANSACTION_COMPLETE == evt_params_cmd_rsp.cmd_status)
            {
              host_setup_state   = ACI_STATUS_TRANSACTION_COMPLETE;
            }
            else
            {
              cur_error_code = ERROR_CMD_RESPONSE_ERROR;
              msg_handle = HANDLE_EVT_ERROR;
            }
          }
          is_send_buffer_free = true;
          lib_aci_post_msg_hook(HANDLE_TRANSACTION_FINISHED);
        }
      }
      break;
    case ACI_EVT_DEVICE_STARTED:
      {
        aci_evt_params_device_started_t evt_params_device_started;
        acil_decode_evt_device_started(&(p_rcvd_evt->buffer[0]), &evt_params_device_started);
        msg_handle = device_mode2handle(evt_params_device_started.device_mode);
        nb_available_credits = evt_params_device_started.credit_available;

        // THIS WAS MISSING
        nb_available_credits_on_device_started = nb_available_credits;
      }
      break;
    case ACI_EVT_DATA_RECEIVED:
      is_request_data_pending = false;
      msg_handle = HANDLE_EVT_RADIO_DATA_RECEIVED;
      break;
    case ACI_EVT_PIPE_STATUS:
      {
        uint8_t i;
        aci_evt_params_pipe_status_t evt_params_pipes_status;
        msg_handle = HANDLE_EVT_RADIO_PIPE_STATUS;

        acil_decode_evt_pipe_status(&(p_rcvd_evt->buffer[0]), &evt_params_pipes_status);
        for (i=0; i < PIPES_ARRAY_SIZE; i++)
        {
          pipes_open_bitmap[i] = evt_params_pipes_status.pipes_open_bitmap[i];
          pipes_closed_bitmap[i] = evt_params_pipes_status.pipes_closed_bitmap[i];
        }
      }
      break;
    case ACI_EVT_DISCONNECTED:
      {
        uint8_t i;
        aci_evt_params_disconnected_t evt_params_disconnected;
        acil_decode_evt_disconnected(&(p_rcvd_evt->buffer[0]), &evt_params_disconnected);
        if (ACI_STATUS_ERROR_ADVT_TIMEOUT == evt_params_disconnected.aci_status)
        {
          msg_handle = HANDLE_EVT_RADIO_ADVERTISE_TIMEDOUT;
        }
        else
        {
          msg_handle = HANDLE_EVT_RADIO_DISCONNECTED;
        }
        for (i=0; i < PIPES_ARRAY_SIZE; i++)
        {
          pipes_open_bitmap[i] = 0;
          pipes_closed_bitmap[i] = 0;
        }
      }
      break;
    case ACI_EVT_DATA_CREDIT:
      {
        aci_evt_params_data_credit_t evt_params_data_credit;
        acil_decode_evt_credit(&(p_rcvd_evt->buffer[0]), &evt_params_data_credit);
        nb_available_credits += evt_params_data_credit.credit;
      }
      msg_handle = HANDLE_EVT_RADIO_CREDIT;
      break;
    case ACI_EVT_TIMING:
      {
        aci_evt_params_timing_t evt_params_timing;
        acil_decode_evt_timing(&(p_rcvd_evt->buffer[0]), &evt_params_timing);
        msg_handle = HANDLE_EVT_RADIO_TIMING;
        cx_rf_interval = evt_params_timing.conn_rf_interval;
      }
      break;
    case ACI_EVT_CONNECTED:
      {
        aci_evt_params_connected_t aci_evt_params_connected;
        acil_decode_evt_connected(&(p_rcvd_evt->buffer[0]), &aci_evt_params_connected);
        msg_handle = HANDLE_EVT_RADIO_CONNECTED;
        cx_rf_interval = aci_evt_params_connected.conn_rf_interval;

        // THIS WAS MISSING
        nb_available_credits = nb_available_credits_on_device_started;
      }
      break;
    case ACI_EVT_BOND_STATUS:
      msg_handle = HANDLE_EVT_RADIO_BOND_STATUS;
      break;
    case ACI_EVT_HW_ERROR:
      msg_handle = HANDLE_EVT_RADIO_HW_ERROR;
      break;
    case ACI_EVT_PIPE_ERROR :
      msg_handle = HANDLE_EVT_RADIO_PIPE_ERROR;
      break;
    case ACI_EVT_ECHO :
      msg_handle = HANDLE_EVT_RADIO_ECHO;
      is_send_buffer_free = true;
      lib_aci_post_msg_hook(HANDLE_TRANSACTION_FINISHED);
      break;
    case ACI_EVT_DATA_ACK :
      msg_handle = HANDLE_EVT_RADIO_DATA_ACK;
      break;
      
    default:
      cur_error_code =  ERROR_RADIO_EVT_UNKNOWN;
      msg_handle = HANDLE_EVT_ERROR;
      break;
    }

    if (msg_handle < NB_MAX_MSG)
    {
      lib_aci_post_msg_hook(msg_handle);
    }
    else
    {
      hal_aci_tl_enable_transaction();
    }
  }
}




/** @} */
