/* Copyright (c) 2011 Nordic Semiconductor. All Rights Reserved.
*
* The information contained herein is property of Nordic Semiconductor ASA.
* Terms and conditions of usage are described in detail in NORDIC
* SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
*
* Licensees are granted free, non-transferable use of the information. NO
* WARRENTY of ANY KIND is provided. This heading must NOT be removed from
* the file.
*
* $LastChangedRevision: 0001 $
*/

/** @file
@brief Implementation of the ANT Message Protocol library.
*/

#include <string.h>  // memcpy
#include <stdio.h>
#include "uart.h"
#include "gpio.h"
#include "lib_ant.h"
#include "LPC11xx.h"
#include "lib_dispatcher.h"
#include "dispatcher_config.h"
#include "system.h"
#include "hal_io.h"




//#define INCLUDE_WAVEFORM

#define MESG_CHANNEL_OFFSET                  3
#define MESG_EVENT_ID_OFFSET                 4
#define MESG_EVENT_CODE_OFFSET               5



// ANT handler defines: if not defined in lib_dispatcher
#ifndef HANDLE_ANT_CMD_RESPONSE_IS_DEFINED
#define HANDLE_ANT_CMD_RESPONSE 				NB_MAX_MSG + 20
#endif
#ifndef HANDLE_ANT_CHANNEL_EVENT_IS_DEFINED
#define HANDLE_ANT_CHANNEL_EVENT				NB_MAX_MSG + 50
#endif
#ifndef HANDLE_ANT_EVENT_RX_BROADCAST_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_BROADCAST			NB_MAX_MSG + 21
#endif
#ifndef HANDLE_ANT_EVENT_RX_ACKNOWLEDGED_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_ACKNOWLEDGED		NB_MAX_MSG + 22
#endif
#ifndef HANDLE_ANT_EVENT_RX_BURST_PACKET_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_BURST_PACKET		NB_MAX_MSG + 23
#endif
#ifndef HANDLE_ANT_EVENT_RX_EXT_BROADCAST_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_EXT_BROADCAST		NB_MAX_MSG + 24
#endif
#ifndef HANDLE_ANT_EVENT_RX_EXT_ACKNOWLEDGED_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_EXT_ACKNOWLEDGED	NB_MAX_MSG + 25
#endif
#ifndef HANDLE_ANT_EVENT_RX_EXT_BURST_PACKET_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_EXT_BURST_PACKET	NB_MAX_MSG + 26
#endif
#ifndef HANDLE_ANT_EVENT_RX_RSSI_BROADCAST_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_RSSI_BROADCAST		NB_MAX_MSG + 27
#endif
#ifndef HANDLE_ANT_EVENT_RX_RSSI_ACKNOWLEDGED_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_RSSI_ACKNOWLEDGED	NB_MAX_MSG + 28
#endif
#ifndef HANDLE_ANT_EVENT_RX_RSSI_BURST_PACKET_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_RSSI_BURST_PACKET	NB_MAX_MSG + 29
#endif
#ifndef HANDLE_ANT_EVENT_RX_BTH_BROADCAST_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_BTH_BROADCAST		NB_MAX_MSG + 30
#endif
#ifndef HANDLE_ANT_EVENT_RX_BTH_ACKNOWLEDGED_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_BTH_ACKNOWLEDGED	NB_MAX_MSG + 31
#endif
#ifndef HANDLE_ANT_EVENT_RX_BTH_BURST_PACKET_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_BTH_BURST_PACKET	NB_MAX_MSG + 32
#endif
#ifndef HANDLE_ANT_EVENT_RX_BTH_EXT_BROADCAST_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_BTH_EXT_BROADCAST	NB_MAX_MSG + 33
#endif
#ifndef HANDLE_ANT_EVENT_RX_BTH_EXT_ACKNOWLEDGED_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_BTH_EXT_ACKNOWLEDGED			NB_MAX_MSG + 34
#endif
#ifndef HANDLE_ANT_EVENT_RX_BTH_EXT_BURST_PACKET_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_BTH_EXT_BURST_PACKET			NB_MAX_MSG + 35
#endif
#ifndef HANDLE_ANT_MSG_NVM_DATA_ID_IS_DEFINED
#define HANDLE_ANT_MSG_NVM_DATA_ID				NB_MAX_MSG + 36
#endif
#ifndef HANDLE_ANT_MESG_NVM_CMD_ID_IS_DEFINED
#define HANDLE_ANT_MESG_NVM_CMD_ID				NB_MAX_MSG + 37
#endif
#ifndef HANDLE_ANT_MESG_INVALID_ID_IS_DEFINED
#define HANDLE_ANT_MESG_INVALID_ID				NB_MAX_MSG + 38
#endif
#ifndef HANDLE_ANT_SYSTEM_STARTUP_IS_DEFINED
#define HANDLE_ANT_SYSTEM_STARTUP				NB_MAX_MSG + 39
#endif
#ifndef HANDLE_ANT_EVENT_TX_IS_DEFINED
#define HANDLE_ANT_EVENT_TX						NB_MAX_MSG + 40
#endif
#ifndef HANDLE_ANT_EVENT_RX_FAIL_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_FAIL				NB_MAX_MSG + 41
#endif
#ifndef HANDLE_ANT_EVENT_RX_SEARCH_TIMEOUT_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_SEARCH_TIMEOUT		NB_MAX_MSG + 42
#endif
#ifndef HANDLE_ANT_EVENT_CHANNEL_CLOSED_IS_DEFINED
#define HANDLE_ANT_EVENT_CHANNEL_CLOSED			NB_MAX_MSG + 43
#endif
#ifndef HANDLE_ANT_UNEXPECTED_EVENT_IS_DEFINED
#define HANDLE_ANT_UNEXPECTED_EVENT				NB_MAX_MSG + 44
#endif
#ifndef HANDLE_ANT_EVENT_TRANSFER_TX_COMPLETED_IS_DEFINED
#define HANDLE_ANT_EVENT_TRANSFER_TX_COMPLETED	NB_MAX_MSG + 46
#endif
#ifndef HANDLE_ANT_EVENT_TRANSFER_TX_FAILED_IS_DEFINED
#define HANDLE_ANT_EVENT_TRANSFER_TX_FAILED		NB_MAX_MSG + 47
#endif
#ifndef HANDLE_ANT_EVENT_RX_FAIL_GO_TO_SEARCH_IS_DEFINED
#define HANDLE_ANT_EVENT_RX_FAIL_GO_TO_SEARCH	NB_MAX_MSG + 48
#endif

#define MAX_CHANNELS ((uint8_t)8)

#define MAX_BURST_MSG_SIZE 256
static uint8_t burst_rx_buffer[MAX_BURST_MSG_SIZE+1];
static uint8_t burst_rx_buffer_index = 0;

static uint8_t channel_rx_buffers[MAX_CHANNELS][MESG_DATA_SIZE];
static uint8_t channel_event_buffers[MAX_CHANNELS][MESG_DATA_SIZE];
static uint8_t cmd_response_buffer[MESG_DATA_SIZE];

static bool open_channels[MAX_CHANNELS];

static bool bInitialized = false;
static volatile bool is_transaction_finished = true;	// SendMessage() should hang if this is false





bool ANT_Init(ushort usBaudRate)
{
	if (bInitialized)
		return false;


	uint8_t br[3];

	switch (usBaudRate) {
	case 1200 :
		br[0] = 0;
		br[1] = 0;
		br[2] = 1;
		break;
	case 2400 :
		br[0] = 0;
		br[1] = 1;
		br[2] = 1;
		break;
	case 4800 :
		br[0] = 0;
		br[1] = 0;
		br[2] = 0;
		break;
	case 9600 :
		br[0] = 1;
		br[1] = 0;
		br[2] = 1;
		break;
	case 19200 :
		br[0] = 0;
		br[1] = 1;
		br[2] = 0;
		break;
	case 38400 :
		br[0] = 1;
		br[1] = 0;
		br[2] = 0;
		break;
	case 50000 :
		br[0] = 1;
		br[1] = 1;
		br[2] = 0;
		break;
	case 57600 :
		br[0] = 1;
		br[1] = 1;
		br[2] = 1;
		break;
	default :
		return false;
	}

	GPIOInit();

	GPIOSetDir( PORT2, 4, 1 ); //BR1/SFLOW
	GPIOSetValue(2,4,br[0]);

	GPIOSetDir( PORT2, 1, 1 ); //BR2/SSCK BAUD 19200
	GPIOSetValue(2,1,br[1]);

	GPIOSetDir( PORT2, 0, 1 ); //BR3
	GPIOSetValue(2,0,br[2]);


	GPIOSetDir( PORT2, 5, 1 ); //SLEEP/MRDY
	GPIOSetValue(2,5,0);

	GPIOSetDir( PORT2, 6, 1 ); //!SUSPEND/SRDY
	GPIOSetValue(2,6,1);

	GPIOSetDir( PORT2, 7, 1 ); // !RESET

	UARTInit((uint32_t)usBaudRate);
	bInitialized = true;

	return true;
}


///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Internal function calls to the serial library.  Each paramerter 
// represents a field in a standard ANT serial message, with ucID as the
// message ID, ucSize as the message size, ucByte0 as the first byte in 
// the message, and the data in pucData as the remaining data bytes.
///////////////////////////////////////////////////////////////////////
static bool SendMessage(uint8_t ucID, uint8_t ucSize, uint8_t ucByte0, uint8_t *pucData)
{
	uint8_t pucOutBuf[ucSize+4];

	pucOutBuf[0] = MESG_TX_SYNC;
	pucOutBuf[1] = ucSize;
	pucOutBuf[2] = ucID;
	pucOutBuf[3] = ucByte0;
	if ((pucData != NULL) && (ucSize > 1))
		memcpy(&pucOutBuf[4], pucData, ucSize - 1);

	pucOutBuf[ucSize+3] = return_XOR(pucOutBuf, ucSize+3);
	while(!is_transaction_finished);	// wait for other transactions
	UARTSend(pucOutBuf, ucSize+4);
	return true;
}



//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to set the search timeout for a particular
// channel on the module
///////////////////////////////////////////////////////////////////////

bool ANT_SetChannelSearchTimeout(uint8_t ucANTChannel, uint8_t ucSearchTimeout)
{

   bool out = SendMessage(MESG_CHANNEL_SEARCH_TIMEOUT_ID, MESG_CHANNEL_SEARCH_TIMEOUT_SIZE,
      ucANTChannel, &ucSearchTimeout);
   is_transaction_finished = false;
   return out;
}


bool ANT_SetChannelLowPrioritySearchTimeout(uint8_t ucANTChannel, uint8_t ucSearchTimeout)
{

   bool out = SendMessage(MESG_SET_LP_SEARCH_TIMEOUT_ID, MESG_SET_LP_SEARCH_TIMEOUT_SIZE,
      ucANTChannel, &ucSearchTimeout);
   is_transaction_finished = false;
   return out;
}

///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to assign a channel
///////////////////////////////////////////////////////////////////////

bool ANT_AssignChannel(uint8_t ucANTChannel, uint8_t ucParam, uint8_t ucNetNumber)
{
   uint8_t aucData[2];

   aucData[0] = ucParam;
   aucData[1] = ucNetNumber;

   bool out = SendMessage(MESG_ASSIGN_CHANNEL_ID, MESG_ASSIGN_CHANNEL_SIZE,
      ucANTChannel, &aucData[0]);
   is_transaction_finished = false;
   return out;
}


bool ANT_AssignChannelExt(uint8_t ucAntChannel, uint8_t chan_type, uint8_t network, uint8_t ext)
{
	uint8_t aucData[3];

	aucData[0] = chan_type;
	aucData[1] = network;
	aucData[2] = ext;

	bool out = SendMessage(MESG_ASSIGN_CHANNEL_ID, MESG_ASSIGN_CHANNEL_SIZE+1,
	  ucAntChannel, &aucData[0]);
	is_transaction_finished = false;
	return out;
}


///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to unassign a channel
///////////////////////////////////////////////////////////////////////

bool ANT_UnAssignChannel(uint8_t ucANTChannel)
{
   bool out = SendMessage(MESG_UNASSIGN_CHANNEL_ID, MESG_UNASSIGN_CHANNEL_SIZE,
      ucANTChannel, NULL);
   is_transaction_finished = false;
   return out;
}

///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to restart ANT on the module
///////////////////////////////////////////////////////////////////////

bool ANT_ResetSystem(void)
{
	 // no reply expected
   return SendMessage(MESG_SYSTEM_RESET_ID, MESG_SYSTEM_RESET_SIZE,
      0, NULL);
}

///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to request a generic message
///////////////////////////////////////////////////////////////////////

bool ANT_RequestMessage(uint8_t ucANTChannel, uint8_t ucMessageID)
{
  bool out = SendMessage(MESG_REQUEST_ID, MESG_REQUEST_SIZE,ucANTChannel, &ucMessageID);
  is_transaction_finished = false;
  return out;
}


///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to set the channel ID
///////////////////////////////////////////////////////////////////////

bool ANT_SetChannelId(uint8_t ucANTChannel, ushort usDeviceNumber, uint8_t ucDeviceType, uint8_t ucManufactureID)
{
   uint8_t aucData[4];

   aucData[0] = (uint8_t)(usDeviceNumber & 0xFF);
   aucData[1] = (uint8_t)((usDeviceNumber >> 8) & 0xFF);
   aucData[2] = ucDeviceType;
   aucData[3] = ucManufactureID;

   bool out = SendMessage(MESG_CHANNEL_ID_ID, MESG_CHANNEL_ID_SIZE, ucANTChannel, &aucData[0]);
   is_transaction_finished = false;
   return out;
}

///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to set the messaging period
///////////////////////////////////////////////////////////////////////

bool ANT_SetChannelPeriod(uint8_t ucANTChannel, ushort usMesgPeriod)
{
   uint8_t aucData[2];

   aucData[0] = (uint8_t)(usMesgPeriod & 0xFF);
   aucData[1] = (uint8_t)((usMesgPeriod >> 8) & 0xFF);

   bool out = SendMessage(MESG_CHANNEL_MESG_PERIOD_ID, MESG_CHANNEL_MESG_PERIOD_SIZE,
      ucANTChannel, &aucData[0]);
   is_transaction_finished = false;
   return out;
}

///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to set the RF frequency for a given channel
//!! This is (should be) a private network function
///////////////////////////////////////////////////////////////////////

bool ANT_SetChannelRFFreq(uint8_t ucANTChannel, uint8_t ucRFFreq)
{
   bool out = SendMessage(MESG_CHANNEL_RADIO_FREQ_ID, MESG_CHANNEL_RADIO_FREQ_SIZE,
      ucANTChannel, &ucRFFreq);
   is_transaction_finished = false;
   return out;
}

bool ANT_SetFrequencyAgilitySettings(uint8_t ant_channel, uint8_t* freq_settings)
{
	bool out = SendMessage(MESG_CONFIG_FREQ_AGILITY_ID, MESG_CONFIG_FREQ_AGILITY_SIZE,
			ant_channel, freq_settings);
	is_transaction_finished = false;
	return out;
}

///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to set the network address for a given
// module channel
//!! This is (should be) a private network function
///////////////////////////////////////////////////////////////////////

bool ANT_SetNetworkKey(uint8_t ucNetNumber, uint8_t *pucKey)
{
   bool out = SendMessage(MESG_NETWORK_KEY_ID, MESG_NETWORK_KEY_SIZE,
      ucNetNumber, pucKey);
   is_transaction_finished = false;
   return out;
}


///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to set the transmit power for the module
///////////////////////////////////////////////////////////////////////

bool ANT_SetTransmitPower(uint8_t ucTransmitPower)
{
   bool out = SendMessage(MESG_RADIO_TX_POWER_ID, MESG_RADIO_TX_POWER_SIZE,
      0, &ucTransmitPower);
   is_transaction_finished = false;
   return out;
}


///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to open an assigned channel
///////////////////////////////////////////////////////////////////////

bool ANT_OpenChannel(uint8_t ucANTChannel)
{
   bool out = SendMessage(MESG_OPEN_CHANNEL_ID, MESG_OPEN_CHANNEL_SIZE,
      ucANTChannel, NULL);
   is_transaction_finished = false;
   return out;
}

///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to close an opend channel
///////////////////////////////////////////////////////////////////////

bool ANT_CloseChannel(uint8_t ucANTChannel)
{
   bool out = SendMessage(MESG_CLOSE_CHANNEL_ID, MESG_CLOSE_CHANNEL_SIZE,
      ucANTChannel, NULL);
   is_transaction_finished = false;
   return out;
}

///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to configure and start CW test mode.
// There is no way to turn off CW mode other than to do a reset on the module.
/////////////////////////////////////////////////////////////////////

bool ANT_InitCWTestMode(void)
{
   bool out = SendMessage(MESG_RADIO_CW_INIT_ID, MESG_RADIO_CW_INIT_SIZE,
      0, NULL);
   is_transaction_finished = false;
   return out;
}

bool ANT_SetCWTestMode(uint8_t ucTransmitPower, uint8_t ucRFChannel)
{
   uint8_t aucData[2];

   aucData[0] = ucTransmitPower;
   aucData[1] = ucRFChannel;

   bool out = SendMessage(MESG_RADIO_CW_MODE_ID, MESG_RADIO_CW_MODE_SIZE,
      0, &aucData[0]);
   is_transaction_finished = false;
   return out;
}


///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to construct and send a broadcast data message.
// This message will be broadcast on the next synchronous channel period.
///////////////////////////////////////////////////////////////////////

bool ANT_SendBroadcastData(uint8_t ucANTChannel, uint8_t *pucData)
{
	// no reply expected
   return SendMessage(MESG_BROADCAST_DATA_ID, MESG_DATA_SIZE,
      ucANTChannel, pucData);
}

///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to construct and send an acknowledged data
// mesg.  This message will be transmitted on the next synchronous channel
// period.
///////////////////////////////////////////////////////////////////////

bool ANT_SendAcknowledgedData(uint8_t ucANTChannel, uint8_t *pucData)
{
	// no reply expected
   return SendMessage(MESG_ACKNOWLEDGED_DATA_ID, MESG_DATA_SIZE,
      ucANTChannel, pucData);
}


///////////////////////////////////////////////////////////////////////
// Priority: Any
//
// Called by the application to construct and send burst data
// mesg.
///////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////
// Used to send burst data.
///////////////////////////////////////////////////////////////////////

bool ANT_SendBurstTransferPacket(uint8_t channel, uint8_t *data, uint8_t size)
{
	// no reply expected
	uint8_t sequence = 0x00;
	uint8_t seq_chan;
	unsigned int i;
	for (i=0; i<size; i+=MESG_DATA_SIZE) {
		seq_chan = sequence | channel;
		if (!SendMessage(MESG_BURST_DATA_ID, (i+MESG_DATA_SIZE > size) ? size-i : MESG_DATA_SIZE, seq_chan, &data[i])) {
			return false;
		}

		if (sequence == SEQUENCE_NUMBER_ROLLOVER)
		{
			sequence = SEQUENCE_NUMBER_INC;
		}
		else
		{
			sequence += SEQUENCE_NUMBER_INC;
		}
		if ((i + 2*MESG_DATA_SIZE) >= size)
		{ // next run is last
			sequence |= SEQUENCE_LAST_MESSAGE;
		}
	}
   return true;
}



//////////////////////////////////////////////////////////////////////
// Serial Processing Code
///////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////
// Priority: Trap
//
// Processes a received serial message.  This funciton is intended to be
// called by the serial message driver code, to be defined by the user,
// when a serial message is received from the ANT module.
// This is run in interrupt context!
///////////////////////////////////////////////////////////////////////
void ANT_SerialHaveMessage(uint8_t *received_data)
{
	 //uint8_t aucData[MESG_DATA_SIZE];
	   uint8_t ucSize;
	   uint8_t ucID;
	   uint8_t ucANTChannel;
	   uint8_t ucANTSequence;

	   //If no response function has been assigned, ignore the message and unlock
	   //the receive buffer
	   /*
	   if (pfResponseFunc == NULL)
	   {
	      return;
	   }*/

	   if (!checkMsg(received_data)) {
		   ENABLE_UART_IRQ();
		   return;
	   }



	   ucSize = received_data[MESG_SIZE_OFFSET];
	   ucID = received_data[MESG_ID_OFFSET];
	   ucANTChannel = received_data[MESG_CHANNEL_OFFSET] & CHANNEL_NUMBER_MASK;
	   ucANTSequence = received_data[MESG_CHANNEL_OFFSET] & SEQUENCE_NUMBER_MASK;


	   //Process the message to determine whether it is a response event or one
	   //of the channel events and call the appropriate event function.
	   uint8_t msg_handle = NB_MAX_MSG;
	   uint8_t* handle_buffer = &channel_rx_buffers[ucANTChannel][0];


	   switch (ucID)
	   {
	      case MESG_RESPONSE_EVENT_ID:
	         if (received_data[MESG_EVENT_ID_OFFSET] != MESG_EVENT_ID) // this is a response
	         {
	        	 if (received_data[MESG_EVENT_ID_OFFSET] == MESG_OPEN_CHANNEL_ID)
	        	 {
	        		 open_channels[ucANTChannel] = true;
	        	 }
				   handle_buffer = &cmd_response_buffer[0];
	               msg_handle = HANDLE_ANT_CMD_RESPONSE;

	         }

	         else // this is an event
	         {
	        	 handle_buffer = &channel_event_buffers[ucANTChannel][0];
	            switch(received_data[MESG_EVENT_CODE_OFFSET])
	            {
	            case EVENT_TX :
	            	msg_handle = HANDLE_ANT_EVENT_TX;
	            	break;
	            case EVENT_RX_FAIL :
	            	msg_handle = HANDLE_ANT_EVENT_RX_FAIL;
	            	break;
	            case EVENT_RX_SEARCH_TIMEOUT :
	            	msg_handle = HANDLE_ANT_EVENT_RX_SEARCH_TIMEOUT;
	            	break;
	            case EVENT_CHANNEL_CLOSED :
	            	open_channels[ucANTChannel] = false;
	            	msg_handle = HANDLE_ANT_EVENT_CHANNEL_CLOSED;
	            	break;
	            case EVENT_TRANSFER_TX_COMPLETED :
	            	msg_handle = HANDLE_ANT_EVENT_TRANSFER_TX_COMPLETED;
	            	break;
	            case EVENT_TRANSFER_TX_FAILED :
	            	msg_handle = HANDLE_ANT_EVENT_TRANSFER_TX_FAILED;
	            	break;
	            case EVENT_RX_FAIL_GO_TO_SEARCH :
	            	msg_handle = HANDLE_ANT_EVENT_RX_FAIL_GO_TO_SEARCH;
	            	break;
	            default :
	            	msg_handle = HANDLE_ANT_UNEXPECTED_EVENT;
	            	break;
	            }

	         }
	         break;

	      case MESG_BROADCAST_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_BROADCAST;
	         break;

	      case MESG_ACKNOWLEDGED_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_ACKNOWLEDGED;
	         break;

	      case MESG_BURST_DATA_ID:
			if (burst_rx_buffer_index + ucSize < MAX_BURST_MSG_SIZE) {
				memcpy(&burst_rx_buffer[burst_rx_buffer_index], &received_data[MESG_DATA_OFFSET+1], ucSize-1);
				burst_rx_buffer_index += ucSize-1;
			}
			if ((ucANTSequence & SEQUENCE_LAST_MESSAGE) == 0x80){ // Last burst packet
				msg_handle = HANDLE_ANT_EVENT_RX_BURST_PACKET;
			}
			else {
				msg_handle = NB_MAX_MSG; // Do not wake application
			}
	         break;

	      case MESG_EXT_BROADCAST_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_EXT_BROADCAST;
	         break;

	      case MESG_EXT_ACKNOWLEDGED_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_EXT_ACKNOWLEDGED;
	         break;

	      case MESG_EXT_BURST_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_EXT_BURST_PACKET;
	         break;

	      case MESG_RSSI_BROADCAST_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_RSSI_BROADCAST;
	         break;

	      case MESG_RSSI_ACKNOWLEDGED_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_RSSI_ACKNOWLEDGED;
	         break;

	      case MESG_RSSI_BURST_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_RSSI_BURST_PACKET;
	         break;

	      case MESG_BTH_BROADCAST_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_BTH_BROADCAST;
	         break;

	      case MESG_BTH_ACKNOWLEDGED_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_BTH_ACKNOWLEDGED;
	         break;

	      case MESG_BTH_BURST_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_BTH_BURST_PACKET;
	         break;

	      case MESG_BTH_EXT_BROADCAST_DATA_ID:
	         break;

	      case MESG_BTH_EXT_ACKNOWLEDGED_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_BTH_EXT_ACKNOWLEDGED;
	         break;

	      case MESG_BTH_EXT_BURST_DATA_ID:
	         msg_handle = HANDLE_ANT_EVENT_RX_BTH_EXT_BURST_PACKET;
	         break;

	      case MESG_NVM_DATA_ID:

				handle_buffer = &cmd_response_buffer[0];
	            msg_handle = HANDLE_ANT_MSG_NVM_DATA_ID;
	            break;

	      case MESG_NVM_CMD_ID:
	    	    handle_buffer = &cmd_response_buffer[0];
	            msg_handle = HANDLE_ANT_MESG_NVM_CMD_ID;

	            break;
	      case MESG_SYSTEM_STARTUP:
	    	  handle_buffer = &cmd_response_buffer[0];
	    	  msg_handle = HANDLE_ANT_SYSTEM_STARTUP;
	    	  break;
	      default:
	    	    handle_buffer = &cmd_response_buffer[0];
	            msg_handle = HANDLE_ANT_MESG_INVALID_ID;
	            break;
	   }
	   if (msg_handle < NB_MAX_MSG)
	   {
		   memcpy(handle_buffer, &received_data[MESG_DATA_OFFSET], ucSize);
		   lib_dispatcher_post_msg(msg_handle, handle_buffer, ucSize, HIGH_PRIORITY);
	   }

	   // re-enable transactions
	   if (!is_transaction_finished)
	   {
		   is_transaction_finished = true;
	   }
	   ENABLE_UART_IRQ();
	   return;
}

//_/_/_/_/_/_/_/_//__/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

uint8_t return_XOR(uint8_t *array, uint8_t length)
{
	uint8_t i;
	uint8_t xor_return = 0;
	for (i = 0; i<length; i++)
	{
		xor_return ^= *(array +i);
	}

	return xor_return;
}

bool checkMsg(uint8_t* msg) {
	if (msg[0] != MESG_TX_SYNC && msg[0] != MESG_RX_SYNC) return false;
	if (msg[msg[1]+3] != return_XOR(msg, msg[1]+3)) return false;
	return true;
}




uint8_t* lib_ant_get_burst_rx_buffer() {
	burst_rx_buffer_index = 0;
	return &burst_rx_buffer[0];
}

bool lib_ant_is_channel_open(uint8_t channel)
{
	return open_channels[channel];
}
