/* Copyright (c) 2010 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision:  $
 */ 
#include "system.h"

#include "hal_io.h"
#include "gpio.h"
#include "hal_aci_tl.h"
#include "LPC11xx.h"
#include "lpc_swu.h"
#include "lib_dispatcher.h"
#include "timer32.h"


#ifndef NULL
#define NULL (void*)0x00
#endif

void system_init(void)
{

	GPIOInit();
	init_timer();

	swu_init(LPC_TMR32B1); // Software uart init

	GPIOSetDir(PORT1, 9, 0);
	GPIOSetInterrupt(PORT1, 9, 0, 0, 1); // radio_active interrupt setup
	GPIOIntEnable(PORT1, 9);

	NVIC_SetPriority(UART_IRQn, 1); 		// Set low priority to HW UART
	NVIC_SetPriority(EINT2_IRQn, 1); 		// Set low priority to HAL_ACI_ON_READY_SIGNAL
	NVIC_SetPriority(EINT1_IRQn, 1); 		// Set low priority to BTLE radio_active signal
	NVIC_SetPriority(TIMER_32_1_IRQn, 0);	// Set high priority to SW_UART timer interrupt

	HAL_IO_CONFIG(HAL_IO_LED0, HAL_IO_OUTPUT, HAL_IO_OUTPUT_NORMAL_STRENGTH);
	HAL_IO_CONFIG(HAL_IO_LED1, HAL_IO_OUTPUT, HAL_IO_OUTPUT_NORMAL_STRENGTH);
	HAL_IO_CONFIG(HAL_IO_LED2, HAL_IO_OUTPUT, HAL_IO_OUTPUT_NORMAL_STRENGTH);
	HAL_IO_CONFIG(HAL_IO_LED3, HAL_IO_OUTPUT, HAL_IO_OUTPUT_NORMAL_STRENGTH);

	HAL_IO_SET_STATE(HAL_IO_LED0, 0);
	HAL_IO_SET_STATE(HAL_IO_LED1, 0);
	HAL_IO_SET_STATE(HAL_IO_LED2, 0);
	HAL_IO_SET_STATE(HAL_IO_LED3, 0);
	
}


void reset_btle_radio()
{

	GPIOSetValue( PORT1, 8, 0 );
	delay_ms(30);
	GPIOSetValue( PORT1, 8, 1 );
	//delay_ms(62);
}

void reset_ant_radio() {
	//if (!bInitialized) return;
	GPIOSetValue(PORT2, 7, 0);
	delay_ms(10);
	GPIOSetValue(PORT2, 7, 1);
	delay_ms(2);

}


void TIMER32_1_IRQHandler(void) // SW UART timer
{
	swu_isr_tx(LPC_TMR32B1);
	swu_isr_rx(LPC_TMR32B1);
}

void PIOINT2_IRQHandler(void) // hal_aci_tl RDYN interrupt
{
  uint32_t regVal;

  regVal = GPIOIntStatus( PORT2, 9 );
  if ( regVal )
  {
	GPIOIntClear( PORT2, 9 );
	hal_aci_tl_on_ready_signal();
  }
  return;

}


