/*
 * weight_app_ant.c
 *
 *  Created on: 16. may 2011
 *      Author: pebr
 */

#include <stdio.h>  // printf/puts
#include "lib_ant.h"
#include "lib_aci.h"
#include "lib_dispatcher.h"
#include "weight_app_ant.h"
#include "LPC11xx.h"
#include "system.h"
#include "hal_io.h"

typedef enum {

	ANT_IDLE = 0,
	ANT_SETUP_SET_CHANNEL_ID,
	ANT_SETUP_SET_CHANNEL_PERIOD,
	ANT_SETUP_SET_CHANNEL_SEARCH_TIMEOUT,
	ANT_SETUP_SET_TX_POWER,
	ANT_SETUP_SET_RADIO_FREQUENCY,
	ANT_SETUP_OPEN_CHANNEL,
	ANT_RUN,
	ANT_FAILURE
} ant_state_t;



ant_state_t state = ANT_IDLE;
ant_state_t next_state;

static bool connect_btle = true;

void on_ant_process_weight_app(uint8_t size, uint8_t* buffer)
{

	switch(state)
	{
	case ANT_IDLE :
		ANT_AssignChannel(ANT_CHANNEL, ant_setup.channel_type, ANT_NET);
		next_state = ANT_SETUP_SET_CHANNEL_ID;
		break;
	case ANT_SETUP_SET_CHANNEL_ID :
		ANT_SetChannelId(ANT_CHANNEL, ANT_DEVICE_NO, ANT_DEVICE_TYPE, ANT_MANUFACTURE_ID);
		next_state = ANT_SETUP_SET_CHANNEL_PERIOD;
		break;
	case ANT_SETUP_SET_CHANNEL_PERIOD :
		ANT_SetChannelPeriod(ANT_CHANNEL, CLOCK_FREQUENCY/ant_setup.channel_period);
		next_state = ANT_SETUP_SET_CHANNEL_SEARCH_TIMEOUT;
		break;
	case ANT_SETUP_SET_CHANNEL_SEARCH_TIMEOUT :
		ANT_SetChannelSearchTimeout(ANT_CHANNEL, ant_setup.search_timeout);
		next_state = ANT_SETUP_SET_TX_POWER;
		break;
	case ANT_SETUP_SET_TX_POWER :
		ANT_SetTransmitPower(ant_setup.tx_power);
		next_state = ANT_SETUP_SET_RADIO_FREQUENCY;
		break;
	case ANT_SETUP_SET_RADIO_FREQUENCY :
		ANT_SetChannelRFFreq(ANT_CHANNEL, ant_setup.frequencies[0]);
		next_state = ANT_SETUP_OPEN_CHANNEL;
		break;
	case ANT_SETUP_OPEN_CHANNEL :
		ANT_OpenChannel(ANT_CHANNEL);
		next_state = ANT_RUN;
		break;
	case ANT_RUN :
		next_state = ANT_RUN;
		break;
	case ANT_FAILURE :
		puts("ANT failed\n");
		break;
	default :
		printf("ant invalid state: %d\n", state);
		break;
	}

}

void on_ant_cmd_response_event(uint8_t size, uint8_t* buffer)
{

	if (buffer[2] != RESPONSE_NO_ERROR)
	{
		printf("ANT cmd 0x%X failed with code 0x%X\n", buffer[1], buffer[2]);
		next_state = ANT_FAILURE;
	}

	state = next_state;
	lib_dispatcher_post_msg(HANDLE_ANT_PROCESS_WEIGHT_APP, NULL, 0, NORMAL_PRIORITY);

}


void on_ant_event_rx_broadcast(uint8_t size, uint8_t* buffer)
{

	static uint8_t weight[2];
	if (connect_btle)
	{
		lib_dispatcher_post_msg(HANDLE_BTLE_PROCESS_WEIGHT_APP, "start", 5, HIGH_PRIORITY);
		connect_btle = false;
	}

	if (buffer[1] == 0x01)
	{
		weight[0] = buffer[8];
		weight[1] = buffer[7];
		services_send_weight_weight(&weight[0], 2);
	}

}



void on_ant_event_rx_fail(uint8_t size, uint8_t* buffer)
{
	puts("ANT rx fail\n");
}

void on_ant_event_rx_fail_go_to_search(uint8_t size, uint8_t* buffer)
{
	puts("ANT goes to search\n");
	connect_btle = true;
	lib_aci_disconnect(ACI_REASON_TERMINATE);
	lib_aci_enable_transmission();
}

void on_ant_unexpected_event(uint8_t size, uint8_t* buffer)
{
	printf("ANT unknown event: 0x%X 0x%X\n", buffer[1], buffer[2]);
}

void on_ant_mesg_invalid_id(uint8_t size, uint8_t* buffer)
{
	uint8_t i;
	puts("ANT Invalid id: ");
	for (i=0; i<size; i++) printf("0x%X ", buffer[i]);
	puts("\n");

}

void on_ant_system_startup(uint8_t size, uint8_t* buffer)
{
	puts("ANT device started.\n");
	state = ANT_IDLE;
	next_state = ANT_IDLE;
	lib_dispatcher_post_msg(HANDLE_ANT_PROCESS_WEIGHT_APP, NULL, 0, NORMAL_PRIORITY);

}

