


#include <stdio.h>
#include <string.h>

#include "hal_io.h"
#include "lib_aci.h"
#include "lib_dispatcher.h"
#include "system.h"
#include "services.h"
#include "weight_app_btle.h"
#include "bool.h"
#include "timer32.h"

static volatile btle_state_t app_state, next_app_state;
static volatile bool connected = false;

void on_transaction_finished(uint8_t size, uint8_t* buffer)
{
	app_state = next_app_state;
	lib_dispatcher_post_msg(HANDLE_BTLE_PROCESS_WEIGHT_APP, NULL, 0, NORMAL_PRIORITY);
}


void on_btle_process_weight_app(uint8_t size, uint8_t* buffer)
{
	static bool connect_cmd_sent = false;

	switch(app_state)
	{
	case COUNTER_APP_SEND_CONFIG_UPLOAD :
		next_app_state = COUNTER_APP_SEND_CONFIG_UPLOAD;
		if (lib_aci_send_setup_msg())
		{
			next_app_state = COUNTER_APP_WAIT_STDBY;
		}
		break;
	case COUNTER_APP_WAIT_STDBY :
		next_app_state = COUNTER_APP_WAIT_STDBY;
		if (!lib_aci_is_host_setup_complete())
		{
			lib_aci_send_setup_msg();
		}
		break;
	case COUNTER_APP_IDLE :
		if (strncmp(&buffer[0], "start", 5) == 0)
		{
			printf("Starting BTLE with: \n\tTx Power: %d\n", btle_setup.tx_power);
			printf("\tAdv timeout: %d\n", btle_setup.adv_timeout);
			puts("Connecting...\n");
			lib_aci_connect(btle_setup.adv_timeout, 320);
		}
		break;
	case COUNTER_APP_RUN :
		//services_update_pipes();
		next_app_state = COUNTER_APP_RUN;
		break;
	default :
		printf("wrong app state: %d\n", app_state);
		break;
	}

	lib_aci_enable_transmission();
}

void on_radio_started_stdby(uint8_t size, uint8_t* buffer)
{
	puts("BTLE started in standby mode\n");
	app_state = COUNTER_APP_IDLE;
	//lib_dispatcher_post_msg(HANDLE_BTLE_PROCESS_WEIGHT_APP, NULL, 0, NORMAL_PRIORITY);
	lib_aci_enable_transmission();
}




void on_radio_started_setup(uint8_t size, uint8_t* buffer)
{

	puts("BTLE started in setup mode\n");
	app_state = COUNTER_APP_SEND_CONFIG_UPLOAD;
	lib_dispatcher_post_msg(HANDLE_BTLE_PROCESS_WEIGHT_APP, NULL, 0, NORMAL_PRIORITY);
	lib_aci_enable_transmission();
}


void on_radio_connected (uint8_t size, uint8_t* buffer)
{
	aci_evt_params_connected_t addr;
	lib_aci_interpret_evt_connected(&addr);
	printf("BTLE connected to 0x%X%X%X%X%X%X\nConnection interval: %d ms\n", addr.dev_addr[5], addr.dev_addr[4], addr.dev_addr[3], addr.dev_addr[2], addr.dev_addr[1], addr.dev_addr[0], addr.conn_rf_interval*5/4);
	HAL_IO_SET_STATE(HAL_IO_LED0, 0);
	HAL_IO_SET_STATE(HAL_IO_LED1, 1);
	connected = true;
	app_state = COUNTER_APP_RUN;
	lib_dispatcher_post_msg(HANDLE_BTLE_PROCESS_WEIGHT_APP, NULL, 0, NORMAL_PRIORITY);
	lib_aci_enable_transmission();
}

void on_disconnect_event(uint8_t size, uint8_t* buffer)
{
	HAL_IO_SET_STATE(HAL_IO_LED0, 0);
	HAL_IO_SET_STATE(HAL_IO_LED1, 0);
	connected = false;
	app_state = COUNTER_APP_IDLE;
	next_app_state = COUNTER_APP_IDLE;
	lib_aci_enable_transmission();
}

void on_error (uint8_t size, uint8_t* buffer)
{

	lib_aci_error_descriptor_t error;
	lib_aci_interpret_error(&error);
	printf("BTLE error: 0x%X 0x%X 0x%X\n", error.error_code, error.error_sub_code1, error.error_sub_code2);
	lib_aci_enable_transmission();
	//while (1)
		//;
}

void on_hw_error (uint8_t size, uint8_t* buffer)
{

	puts("BTLE hw error!\n");

	HAL_IO_SET_STATE(HAL_IO_LED0, 1);
	HAL_IO_SET_STATE(HAL_IO_LED1, 1);
	lib_aci_enable_transmission();
	while (1)
		;
}

void on_btle_pipe_error(uint8_t size, uint8_t* buffer)
{
	aci_evt_params_pipe_error_t error;
	lib_aci_interpret_evt_pipe_error(&error);
	printf("BTLE pipe no %d error. Code 0x%x\n", error.pipe_number, error.error_code);
	lib_aci_enable_transmission();
}


void on_advertise_timeout(uint8_t size, uint8_t* buffer)
{
	puts("BTLE advertising timedout.\n");
	app_state = COUNTER_APP_IDLE;
	next_app_state = COUNTER_APP_IDLE;
	lib_aci_enable_transmission();
}


/*
void PIOINT1_IRQHandler(void)
{
	// Active signal interrupt handler
	uint32_t regVal;
	regVal = GPIOIntStatus(PORT1, 9);
	static uint8_t toggle = 0;
	if (regVal) {
		GPIOIntClear(PORT1, 9);

		if (connected && lib_aci_is_pipe_available(PIPE_PACKET_COUNTER_COUNTER_TX))
		{
			if (lib_aci_send_data(PIPE_PACKET_COUNTER_COUNTER_TX, &btle_counter, 1))
			{
				sent_packets++;
				btle_counter++;
			}
		}

		HAL_IO_SET_STATE(HAL_IO_LED0, toggle);
		toggle = !toggle;
	}
}
*/
